package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.advancements.critereon.EntityPredicate
import net.minecraft.advancements.critereon.InventoryChangeTrigger
import net.minecraft.advancements.critereon.ItemPredicate
import net.minecraft.advancements.critereon.MinMaxBounds
import net.minecraft.data.DataGenerator
import net.minecraft.data.recipes.FinishedRecipe
import net.minecraft.data.recipes.RecipeProvider
import net.minecraft.resources.ResourceLocation
import net.minecraft.tags.Tag
import net.minecraft.world.item.Item
import net.minecraft.world.item.crafting.Ingredient
import net.minecraft.world.level.ItemLike
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.recipe.PlatePressRecipe
import java.util.function.Consumer

private typealias RecipeLambda = (MatteryRecipeProvider, Consumer<FinishedRecipe>) -> Unit

private fun has(p_176521_: MinMaxBounds.Ints, p_176522_: ItemLike): InventoryChangeTrigger.TriggerInstance {
	return inventoryTrigger(ItemPredicate.Builder.item().of(p_176522_).withCount(p_176521_).build())
}

fun has(p_125978_: ItemLike): InventoryChangeTrigger.TriggerInstance {
	return inventoryTrigger(ItemPredicate.Builder.item().of(p_125978_).build())
}

fun has(p_125976_: Tag<Item>): InventoryChangeTrigger.TriggerInstance {
	return inventoryTrigger(ItemPredicate.Builder.item().of(p_125976_).build())
}

fun inventoryTrigger(vararg p_126012_: ItemPredicate): InventoryChangeTrigger.TriggerInstance {
	return InventoryChangeTrigger.TriggerInstance(
		EntityPredicate.Composite.ANY,
		MinMaxBounds.Ints.ANY,
		MinMaxBounds.Ints.ANY,
		MinMaxBounds.Ints.ANY,
		p_126012_
	)
}

class MatteryRecipeProvider(generatorIn: DataGenerator) : RecipeProvider(generatorIn) {
	private val lambdas = ArrayList<RecipeLambda>()

	fun lambda(lambda: RecipeLambda) = lambdas.add(lambda)

	override fun buildCraftingRecipes(callback: Consumer<FinishedRecipe>) {
		for (lambda in lambdas) {
			lambda(this, callback)
		}
	}

	fun plate(id: String, count: Int = 1, workTicks: Int = 200) {
		lambda { it, callback ->
			callback.accept(PlatePressShallowFinishedRecipe(
				ResourceLocation(DataGen.MOD_ID, "plates/$id"),
				ResourceLocation("forge", "ingots/$id"),
				ResourceLocation("forge", "plates/$id"),
				count,
				workTicks
			))
		}
	}

	fun plate(id: String, ingredient: Ingredient, result: Ingredient, count: Int = 1, workTicks: Int = 200) {
		lambda { it, callback ->
			callback.accept(PlatePressFinishedRecipe(PlatePressRecipe(ResourceLocation(DataGen.MOD_ID, "plate_$id"), ingredient, result, count, workTicks)))
		}
	}
}
