package ru.dbotthepony.mc.otm;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.Level;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.TickEvent;
import net.minecraftforge.event.server.ServerAboutToStartEvent;
import net.minecraftforge.event.server.ServerStoppingEvent;
import net.minecraftforge.eventbus.api.EventPriority;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.LogicalSide;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import ru.dbotthepony.mc.otm.block.entity.blackhole.ExplosionQueue;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.drive.DrivePool;
import ru.dbotthepony.mc.otm.client.AndroidGui;
import ru.dbotthepony.mc.otm.client.EventHandler;
import ru.dbotthepony.mc.otm.client.model.GravitationStabilizerModel;
import ru.dbotthepony.mc.otm.client.model.TritaniumArmorModel;
import ru.dbotthepony.mc.otm.core.ImpreciseFraction;
import ru.dbotthepony.mc.otm.item.ItemPortableCondensationDrive;
import ru.dbotthepony.mc.otm.matter.MatterDataKt;
import ru.dbotthepony.mc.otm.matter.MatterRegistryKt;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.registry.*;
import ru.dbotthepony.mc.otm.storage.ItemStackWrapper;
import ru.dbotthepony.mc.otm.storage.StorageObjectRegistry;
import ru.dbotthepony.mc.otm.storage.StorageObjectTuple;

import javax.annotation.ParametersAreNonnullByDefault;
import java.util.ArrayList;
import java.util.WeakHashMap;
import java.util.function.Consumer;
import java.util.function.Supplier;

// The value here should match an entry in the META-INF/mods.toml file
@Mod(OverdriveThatMatters.MOD_ID)
@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public final class OverdriveThatMatters {
	// Directly reference a log4j logger.
	public static final String MOD_ID = "overdrive_that_matters";
	public static final Logger LOGGER = LogManager.getLogger();

	public static OverdriveThatMatters INSTANCE;
	public AndroidGui ANDROID_GUI;
	public StorageObjectTuple<ItemStackWrapper> ITEM_STORAGE;

	public final CreativeModeTab CREATIVE_TAB = new CreativeModeTab("otm") {
		@Override
		public ItemStack makeIcon() {
			return new ItemStack(MItems.BATTERY_CREATIVE, 1);
		}
	};

	public OverdriveThatMatters() {
		INSTANCE = this;

		// Register the setup method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setup);
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setupClient);

		// Register ourselves for server and other game events we are interested in
		MinecraftForge.EVENT_BUS.register(this);
		MinecraftForge.EVENT_BUS.register(TickerKt.class);
		MinecraftForge.EVENT_BUS.register(AndroidCapabilityPlayer.Companion);
		MinecraftForge.EVENT_BUS.register(AndroidCapability.Companion);
		MinecraftForge.EVENT_BUS.register(MatterRegistryKt.class);
		MinecraftForge.EVENT_BUS.register(MatterDataKt.class);
		MinecraftForge.EVENT_BUS.register(ExplosionQueue.Companion);

		FMLJavaModLoadingContext.get().getModEventBus().register(MatteryCapability.class);

		FMLJavaModLoadingContext.get().getModEventBus().register(MItems.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(MBlocks.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(MBlockEntities.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(MMenus.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(AndroidFeatures.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(AndroidResearch.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(MStats.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(MRecipes.class);

		MinecraftForge.EVENT_BUS.register(DrivePool.INSTANCE);
		MinecraftForge.EVENT_BUS.register(ItemPortableCondensationDrive.Companion);

		FMLJavaModLoadingContext.get().getModEventBus().addListener(AndroidCapability::registerEffects);

		// force Registry static initializer to be called
		Registry.dummy();
	}

	private void setup(final FMLCommonSetupEvent event) {
		MatteryNetworking.register();
		// LOGGER.info("Registered network");

		ITEM_STORAGE = StorageObjectRegistry.register(ItemStackWrapper.class, ItemStackWrapper.EMPTY, new ImpreciseFraction("3.125"));
	}

	private void setupClient(final FMLClientSetupEvent event) {
		ANDROID_GUI = new AndroidGui();
		MinecraftForge.EVENT_BUS.register(ANDROID_GUI);
		MinecraftForge.EVENT_BUS.register(AndroidGui.class);
		MinecraftForge.EVENT_BUS.register(EventHandler.class);

		TritaniumArmorModel.register();
		GravitationStabilizerModel.register();
	}
}
