package ru.dbotthepony.mc.otm.android;

import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraftforge.registries.ForgeRegistryEntry;
import ru.dbotthepony.mc.otm.registry.Registry;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;

import java.util.ArrayList;
import java.util.List;

public class AndroidResearchType<T extends AndroidResearch> extends ForgeRegistryEntry<AndroidResearchType<?>> {
	/**
	 * @return List of research required to be completed before this is unlocked
	 */
	public List<AndroidResearchType<?>> getPrerequisites() {
		return List.of();
	}

	private List<AndroidResearchType<?>> blocking;
	private List<AndroidResearchType<?>> unlocks;

	/**
	 * @return List of research blocked once this research is unlocked
	 */
	public List<AndroidResearchType<?>> getBlocking() {
		if (blocking == null) {
			var list = new ArrayList<AndroidResearchType<?>>();

			for (var type : Registry.ANDROID_RESEARCH) {
				if (type.getBlockedBy().contains(this)) {
					list.add(type);
				}
			}

			blocking = List.copyOf(list);
		}

		return blocking;
	}

	/**
	 * @return List of research unlocked once this research is unlocked
	 */
	public List<AndroidResearchType<?>> getUnlocks() {
		if (unlocks == null) {
			var list = new ArrayList<AndroidResearchType<?>>();

			for (var type : Registry.ANDROID_RESEARCH) {
				if (type.getPrerequisites().contains(this)) {
					list.add(type);
				}
			}

			unlocks = List.copyOf(list);
		}

		return unlocks;
	}

	/**
	 * @return List of research once unlocked blocks this research
	 */
	public List<AndroidResearchType<?>> getBlockedBy() {
		return List.of();
	}

	public interface AndroidResearchFactory<T extends AndroidResearch> {
		T factory(AndroidResearchType<T> entry, AndroidCapabilityPlayer capability);
	}

	protected final AndroidResearchFactory<T> factory;

	public AndroidResearchType(AndroidResearchFactory<T> factory) {
		this.factory = factory;
	}

	public T factory(AndroidCapabilityPlayer capability) {
		return factory.factory(this, capability);
	}

	public Component getDisplayName() {
		if (getRegistryName() == null)
			return new TranslatableComponent("android_research.null.null");

		return new TranslatableComponent("android_research." + getRegistryName().getNamespace() + "." + getRegistryName().getPath());
	}
}
