package ru.dbotthepony.mc.otm.menu;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.FormattedText;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.network.chat.TranslatableComponent;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.core.Fraction;
import ru.dbotthepony.mc.otm.core.ImpreciseFraction;

import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class FormattingHelper {
	public static final TranslatableComponent SUFFIX_KILO = new TranslatableComponent("otm.suffix_raw.kilo");
	public static final TranslatableComponent SUFFIX_MEGA = new TranslatableComponent("otm.suffix_raw.mega");
	public static final TranslatableComponent SUFFIX_GIGA = new TranslatableComponent("otm.suffix_raw.giga");
	public static final TranslatableComponent SUFFIX_TERA = new TranslatableComponent("otm.suffix_raw.tera");
	public static final TranslatableComponent SUFFIX_PETA = new TranslatableComponent("otm.suffix_raw.peta");
	public static final TranslatableComponent SUFFIX_EXA = new TranslatableComponent("otm.suffix_raw.exa");
	public static final TranslatableComponent SUFFIX_ZETTA = new TranslatableComponent("otm.suffix_raw.zetta");
	public static final TranslatableComponent SUFFIX_YOTTA = new TranslatableComponent("otm.suffix_raw.yotta");

	public static final TranslatableComponent SUFFIX_DECI = new TranslatableComponent("otm.suffix_raw.deci");
	public static final TranslatableComponent SUFFIX_CENTI = new TranslatableComponent("otm.suffix_raw.centi");
	public static final TranslatableComponent SUFFIX_MILLI = new TranslatableComponent("otm.suffix_raw.milli");
	public static final TranslatableComponent SUFFIX_MICRO = new TranslatableComponent("otm.suffix_raw.micro");
	public static final TranslatableComponent SUFFIX_NANO = new TranslatableComponent("otm.suffix_raw.nano");
	public static final TranslatableComponent SUFFIX_PICO = new TranslatableComponent("otm.suffix_raw.pico");
	public static final TranslatableComponent SUFFIX_FEMTO = new TranslatableComponent("otm.suffix_raw.femto");
	public static final TranslatableComponent SUFFIX_ATTO = new TranslatableComponent("otm.suffix_raw.atto");
	public static final TranslatableComponent SUFFIX_ZEPTO = new TranslatableComponent("otm.suffix_raw.zepto");
	public static final TranslatableComponent SUFFIX_YOCTO = new TranslatableComponent("otm.suffix_raw.yocto");

	public static final List<TranslatableComponent> BIG_SUFFIX_STRIPE = List.of(
		SUFFIX_KILO,
		SUFFIX_MEGA,
		SUFFIX_GIGA,
		SUFFIX_TERA,
		SUFFIX_PETA,
		SUFFIX_EXA,
		SUFFIX_ZETTA,
		SUFFIX_YOTTA
	);

	public static final List<TranslatableComponent> SMALL_SUFFIX_STRIPE = List.of(
		SUFFIX_DECI,
		SUFFIX_CENTI,
		SUFFIX_MILLI,
		SUFFIX_MICRO,
		SUFFIX_NANO,
		SUFFIX_PICO,
		SUFFIX_FEMTO,
		SUFFIX_ATTO,
		SUFFIX_ZEPTO,
		SUFFIX_YOCTO
	);

	public static final List<TranslatableComponent> BIG_SUFFIX_STRIPE_INV;
	public static final List<TranslatableComponent> SMALL_SUFFIX_STRIPE_INV;

	static {
		var list = new ArrayList<TranslatableComponent>(BIG_SUFFIX_STRIPE.size());

		int i2 = 0;
		for (int i = BIG_SUFFIX_STRIPE.size() - 1; i >= 0; i--) {
			list.add(BIG_SUFFIX_STRIPE.get(i2));
			i2++;
		}

		BIG_SUFFIX_STRIPE_INV = List.copyOf(list);
	}

	static {
		var list = new ArrayList<TranslatableComponent>(BIG_SUFFIX_STRIPE.size());

		int i2 = 0;
		for (int i = BIG_SUFFIX_STRIPE.size() - 1; i >= 0; i--) {
			list.add(BIG_SUFFIX_STRIPE.get(i2));
			i2++;
		}

		SMALL_SUFFIX_STRIPE_INV = List.copyOf(list);
	}

	public static final String[] SUFFIX_COMPONENTS_ABOVE_ONE = new String[] {
		"otm.suffix.kilo",
		"otm.suffix.mega",
		"otm.suffix.giga",
		"otm.suffix.tera",
		"otm.suffix.peta",
		"otm.suffix.exa",
		"otm.suffix.zetta",
		"otm.suffix.yotta",
	};

	public static final BigDecimal[] SUFFIX_ABOVE_ONE = new BigDecimal[] {
		new BigDecimal("1000"), // "otm.suffix.kilo": "k%s",
		new BigDecimal("1000000"), // "otm.suffix.mega": "M%s",
		new BigDecimal("1000000000"), // "otm.suffix.giga": "G%s",
		new BigDecimal("1000000000000"), // "otm.suffix.tera": "T%s",
		new BigDecimal("1000000000000000"), // "otm.suffix.peta": "P%s",
		new BigDecimal("1000000000000000000"), // "otm.suffix.exa": "E%s",
		new BigDecimal("1000000000000000000000"), // "otm.suffix.zetta": "Z%s",
		new BigDecimal("1000000000000000000000000"), // "otm.suffix.yotta": "Y%s",
	};

	public static final Fraction[] SUFFIX_ABOVE_ONE_FRAC = new Fraction[] {
		Fraction.fromString("1000"), // "otm.suffix.kilo": "k%s",
		Fraction.fromString("1000000"), // "otm.suffix.mega": "M%s",
		Fraction.fromString("1000000000"), // "otm.suffix.giga": "G%s",
		Fraction.fromString("1000000000000"), // "otm.suffix.tera": "T%s",
		Fraction.fromString("1000000000000000"), // "otm.suffix.peta": "P%s",
		Fraction.fromString("1000000000000000000"), // "otm.suffix.exa": "E%s",
		Fraction.fromString("1000000000000000000000"), // "otm.suffix.zetta": "Z%s",
		Fraction.fromString("1000000000000000000000000"), // "otm.suffix.yotta": "Y%s",
	};

	public static final ImpreciseFraction[] SUFFIX_ABOVE_ONE_IM_FRAC = new ImpreciseFraction[] {
		new ImpreciseFraction("1000"), // "otm.suffix.kilo": "k%s",
		new ImpreciseFraction("1000000"), // "otm.suffix.mega": "M%s",
		new ImpreciseFraction("1000000000"), // "otm.suffix.giga": "G%s",
		new ImpreciseFraction("1000000000000"), // "otm.suffix.tera": "T%s",
		new ImpreciseFraction("1000000000000000"), // "otm.suffix.peta": "P%s",
		new ImpreciseFraction("1000000000000000000"), // "otm.suffix.exa": "E%s",
		new ImpreciseFraction("1000000000000000000000"), // "otm.suffix.zetta": "Z%s",
		new ImpreciseFraction("1000000000000000000000000"), // "otm.suffix.yotta": "Y%s",
	};

	public static final String[] SUFFIX_COMPONENTS_BELOW_ONE = new String[] {
		"otm.suffix.deci",
		"otm.suffix.centi",
		"otm.suffix.milli",
		"otm.suffix.micro",
		"otm.suffix.nano",
		"otm.suffix.pico",
		"otm.suffix.femto",
		"otm.suffix.atto",
		"otm.suffix.zepto",
		"otm.suffix.yocto",
	};

	public static final BigDecimal[] SUFFIX_BELOW_ONE = new BigDecimal[] {
		new BigDecimal("0.1"), // "otm.suffix.milli": "d%s",
		new BigDecimal("0.01"), // "otm.suffix.milli": "c%s",
		new BigDecimal("0.001"), // "otm.suffix.milli": "m%s",
		new BigDecimal("0.000001"), // "otm.suffix.micro": "μ%s",
		new BigDecimal("0.000000001"), // "otm.suffix.nano": "n%s",
		new BigDecimal("0.000000000001"), // "otm.suffix.pico": "p%s",
		new BigDecimal("0.000000000000001"), // "otm.suffix.femto": "f%s",
		new BigDecimal("0.000000000000000001"), // "otm.suffix.atto": "a%s",
		new BigDecimal("0.000000000000000000001"), // "otm.suffix.zepto": "z%s",
		new BigDecimal("0.000000000000000000000001"), // "otm.suffix.yocto": "y%s",
	};

	public static final Fraction[] SUFFIX_BELOW_ONE_FRAC = new Fraction[] {
		Fraction.fromString("0.1"), // "otm.suffix.milli": "d%s",
		Fraction.fromString("0.01"), // "otm.suffix.milli": "c%s",
		Fraction.fromString("0.001"), // "otm.suffix.milli": "m%s",
		Fraction.fromString("0.000001"), // "otm.suffix.micro": "μ%s",
		Fraction.fromString("0.000000001"), // "otm.suffix.nano": "n%s",
		Fraction.fromString("0.000000000001"), // "otm.suffix.pico": "p%s",
		Fraction.fromString("0.000000000000001"), // "otm.suffix.femto": "f%s",
		Fraction.fromString("0.000000000000000001"), // "otm.suffix.atto": "a%s",
		Fraction.fromString("0.000000000000000000001"), // "otm.suffix.zepto": "z%s",
		Fraction.fromString("0.000000000000000000000001"), // "otm.suffix.yocto": "y%s",
	};

	public static final ImpreciseFraction[] SUFFIX_BELOW_ONE_IM_FRAC = new ImpreciseFraction[] {
		new ImpreciseFraction("0.1"), // "otm.suffix.milli": "d%s",
		new ImpreciseFraction("0.01"), // "otm.suffix.milli": "c%s",
		new ImpreciseFraction("0.001"), // "otm.suffix.milli": "m%s",
		new ImpreciseFraction("0.000001"), // "otm.suffix.micro": "μ%s",
		new ImpreciseFraction("0.000000001"), // "otm.suffix.nano": "n%s",
		new ImpreciseFraction("0.000000000001"), // "otm.suffix.pico": "p%s",
		new ImpreciseFraction("0.000000000000001"), // "otm.suffix.femto": "f%s",
		new ImpreciseFraction("0.000000000000000001"), // "otm.suffix.atto": "a%s",
		new ImpreciseFraction("0.000000000000000000001"), // "otm.suffix.zepto": "z%s",
		new ImpreciseFraction("0.000000000000000000000001"), // "otm.suffix.yocto": "y%s",
	};

	public static String formatDecimal(BigDecimal value, int decimals) {
		return value.setScale(decimals, RoundingMode.HALF_UP).toString();
	}

	public static final TranslatableComponent POWER_NAME = new TranslatableComponent("otm.gui.power.name");
	public static final TranslatableComponent MATTER_NAME = new TranslatableComponent("otm.gui.matter.name");

	public static Component formatPowerLevel(BigDecimal power, BigDecimal max_power) {
		return new TranslatableComponent("otm.gui.level", formatSI(power, POWER_NAME), formatSI(max_power, POWER_NAME));
	}

	public static Component formatPowerLevel(Fraction power, Fraction max_power) {
		return new TranslatableComponent("otm.gui.level", formatSI(power, POWER_NAME), formatSI(max_power, POWER_NAME));
	}

	public static Component formatPowerLevel(ImpreciseFraction power, ImpreciseFraction max_power) {
		return new TranslatableComponent("otm.gui.level", formatSI(power, POWER_NAME), formatSI(max_power, POWER_NAME));
	}

	public static Component formatMatterLevel(BigDecimal power, BigDecimal max_power) {
		return new TranslatableComponent("otm.gui.level", formatMatterValuePlain(power), formatMatterValuePlain(max_power));
	}

	public static Component formatMatterLevel(Fraction power, Fraction max_power) {
		return new TranslatableComponent("otm.gui.level", formatMatterValuePlain(power), formatMatterValuePlain(max_power));
	}

	public static Component formatMatterLevel(ImpreciseFraction power, ImpreciseFraction max_power) {
		return new TranslatableComponent("otm.gui.level", formatMatterValuePlain(power), formatMatterValuePlain(max_power));
	}

	public static TranslatableComponent formatMatterValue(BigDecimal matter) {
		return new TranslatableComponent("otm.gui.matter.format", formatSI(matter, MATTER_NAME));
	}

	public static TranslatableComponent formatMatterValue(Fraction matter) {
		return new TranslatableComponent("otm.gui.matter.format", formatSI(matter, MATTER_NAME));
	}

	public static TranslatableComponent formatMatterValue(ImpreciseFraction matter) {
		return new TranslatableComponent("otm.gui.matter.format", formatSI(matter, MATTER_NAME));
	}

	public static Component formatMatterValuePlain(BigDecimal matter) {
		return formatSI(matter, MATTER_NAME);
	}

	public static Component formatMatterValuePlain(Fraction matter) {
		return formatSI(matter, MATTER_NAME);
	}

	public static Component formatMatterValuePlain(ImpreciseFraction matter) {
		return formatSI(matter, MATTER_NAME);
	}

	public static Component formatPower(BigDecimal power) {
		return formatSI(power, POWER_NAME);
	}

	public static Component formatPower(Fraction power) {
		return formatSI(power, POWER_NAME);
	}

	public static Component formatPower(ImpreciseFraction power) {
		return formatSI(power, POWER_NAME);
	}

	public static Component formatSI(BigDecimal value) {
		return formatSI(value, "");
	}

	public static Component formatSI(Fraction value) {
		return formatSI(value, "");
	}

	public static Component formatSI(BigDecimal value, Object end_suffix) {
		if (value.compareTo(BigDecimal.ZERO) == 0) {
			if ("".equals(end_suffix)) {
				return new TextComponent("0.00");
			} else {
				return new TranslatableComponent("otm.suffix.merge", "0.00", end_suffix);
			}
		}

		int compare = value.compareTo(BigDecimal.ONE);

		if (compare == 0) {
			if ("".equals(end_suffix)) {
				return new TextComponent("1.00");
			} else {
				return new TranslatableComponent("otm.suffix.merge", "1.00", end_suffix);
			}
		} else if (compare > 0) {
			for (int i = SUFFIX_ABOVE_ONE.length - 1; i >= 0; i--) {
				if (value.compareTo(SUFFIX_ABOVE_ONE[i]) >= 0) {
					return new TranslatableComponent(SUFFIX_COMPONENTS_ABOVE_ONE[i], formatDecimal(value.divide(SUFFIX_ABOVE_ONE[i]), 2), end_suffix);
				}
			}

			if ("".equals(end_suffix)) {
				return new TextComponent(formatDecimal(value, 2));
			} else {
				return new TranslatableComponent("otm.suffix.merge", formatDecimal(value, 2), end_suffix);
			}
		}

		for (int i = SUFFIX_BELOW_ONE.length - 1; i >= 0; i--) {
			if (value.compareTo(SUFFIX_BELOW_ONE[i]) < 0) {
				return new TranslatableComponent(SUFFIX_COMPONENTS_BELOW_ONE[i], formatDecimal(value.divide(SUFFIX_BELOW_ONE[i]), 2), end_suffix);
			}
		}

		if ("".equals(end_suffix)) {
			return new TextComponent(formatDecimal(value, 2));
		} else {
			return new TranslatableComponent("otm.suffix.merge", formatDecimal(value, 2), end_suffix);
		}
	}

	public static Component formatSI(Fraction value, Object end_suffix) {
		if (value.compareTo(Fraction.ZERO) == 0) {
			if ("".equals(end_suffix)) {
				return new TextComponent("0.00");
			} else {
				return new TranslatableComponent("otm.suffix.merge", "0.00", end_suffix);
			}
		}

		int compare = value.compareTo(Fraction.ONE);

		if (compare == 0) {
			if ("".equals(end_suffix)) {
				return new TextComponent("1.00");
			} else {
				return new TranslatableComponent("otm.suffix.merge", "1.00", end_suffix);
			}
		} else if (compare > 0) {
			for (int i = SUFFIX_ABOVE_ONE_FRAC.length - 1; i >= 0; i--) {
				if (value.compareTo(SUFFIX_ABOVE_ONE_FRAC[i]) >= 0) {
					return new TranslatableComponent(SUFFIX_COMPONENTS_ABOVE_ONE[i], value.div(SUFFIX_ABOVE_ONE_FRAC[i]).decimalString(2, true), end_suffix);
				}
			}

			if ("".equals(end_suffix)) {
				return new TextComponent(value.decimalString(2, true));
			} else {
				return new TranslatableComponent("otm.suffix.merge", value.decimalString(2, true), end_suffix);
			}
		}

		for (int i = SUFFIX_BELOW_ONE.length - 1; i >= 0; i--) {
			if (value.compareTo(SUFFIX_BELOW_ONE_FRAC[i]) < 0) {
				return new TranslatableComponent(SUFFIX_COMPONENTS_BELOW_ONE[i], value.div(SUFFIX_BELOW_ONE_FRAC[i]).decimalString(2, true), end_suffix);
			}
		}

		if ("".equals(end_suffix)) {
			return new TextComponent(value.decimalString(2, true));
		} else {
			return new TranslatableComponent("otm.suffix.merge", value.decimalString(2, true), end_suffix);
		}
	}

	public static Component formatSI(ImpreciseFraction value, Object end_suffix) {
		if (value.compareTo(ImpreciseFraction.ZERO) == 0) {
			if ("".equals(end_suffix)) {
				return new TextComponent("0.00");
			} else {
				return new TranslatableComponent("otm.suffix.merge", "0.00", end_suffix);
			}
		}

		int compare = value.compareTo(ImpreciseFraction.ONE);

		if (compare == 0) {
			if ("".equals(end_suffix)) {
				return new TextComponent("1.00");
			} else {
				return new TranslatableComponent("otm.suffix.merge", "1.00", end_suffix);
			}
		} else if (compare > 0) {
			for (int i = SUFFIX_ABOVE_ONE_IM_FRAC.length - 1; i >= 0; i--) {
				if (value.compareTo(SUFFIX_ABOVE_ONE_IM_FRAC[i]) >= 0) {
					return new TranslatableComponent(SUFFIX_COMPONENTS_ABOVE_ONE[i], value.div(SUFFIX_ABOVE_ONE_IM_FRAC[i]).toString(2), end_suffix);
				}
			}

			if ("".equals(end_suffix)) {
				return new TextComponent(value.toString(2));
			} else {
				return new TranslatableComponent("otm.suffix.merge", value.toString(2), end_suffix);
			}
		}

		for (int i = SUFFIX_BELOW_ONE_IM_FRAC.length - 1; i >= 0; i--) {
			if (value.compareTo(SUFFIX_BELOW_ONE_IM_FRAC[i]) < 0) {
				return new TranslatableComponent(SUFFIX_COMPONENTS_BELOW_ONE[i], value.div(SUFFIX_BELOW_ONE_IM_FRAC[i]).toString(2), end_suffix);
			}
		}

		if ("".equals(end_suffix)) {
			return new TextComponent(value.toString(2));
		} else {
			return new TranslatableComponent("otm.suffix.merge", value.toString(2), end_suffix);
		}
	}
}
