package ru.dbotthepony.mc.otm.network;

import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.level.ServerPlayer;
import net.minecraftforge.network.NetworkDirection;
import net.minecraftforge.network.NetworkRegistry;
import net.minecraftforge.network.PacketDistributor;
import net.minecraftforge.network.simple.SimpleChannel;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.block.entity.EnergyCounterPacket;
import ru.dbotthepony.mc.otm.matter.RegistryPacketClear;
import ru.dbotthepony.mc.otm.matter.RegistryPacketFullUpdate;
import ru.dbotthepony.mc.otm.matter.RegistryPacketRemove;
import ru.dbotthepony.mc.otm.matter.RegistryPacketUpdate;
import ru.dbotthepony.mc.otm.menu.MenuDriveViewer;
import ru.dbotthepony.mc.otm.menu.data.*;
import ru.dbotthepony.mc.otm.network.android.*;

import java.util.Optional;

public class MatteryNetworking {
	private static final String PROTOCOL_VERSION = "2";

	private static int next_network_id = 0;

	public static final SimpleChannel CHANNEL = NetworkRegistry.newSimpleChannel(
		new ResourceLocation(OverdriveThatMatters.MOD_ID, "main"),
		() -> PROTOCOL_VERSION,
		PROTOCOL_VERSION::equals,
		PROTOCOL_VERSION::equals
	);

	public static void send(ServerPlayer ply, Object ...messages) {
		if (ply == null) {
			for (var message : messages) {
				CHANNEL.sendToServer(message);
			}

			return;
		}

		var supplier = PacketDistributor.PLAYER.with(() -> ply);

		for (var message : messages) {
			CHANNEL.send(supplier, message);
		}
	}

	public static void register() {
		CHANNEL.registerMessage(
			next_network_id++,
			AndroidBatteryPacket.class,
			AndroidBatteryPacket::write,
			AndroidBatteryPacket::read,
			AndroidBatteryPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidEnergyPacket.class,
			AndroidEnergyPacket::write,
			AndroidEnergyPacket::read,
			AndroidEnergyPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidStatusPacket.class,
			AndroidStatusPacket::write,
			AndroidStatusPacket::read,
			AndroidStatusPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidFeaturePacket.class,
			AndroidFeaturePacket::write,
			AndroidFeaturePacket::read,
			AndroidFeaturePacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			PatternGridPacket.class,
			PatternGridPacket::write,
			PatternGridPacket::read,
			PatternGridPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			MatterTaskPacket.class,
			MatterTaskPacket::write,
			MatterTaskPacket::read,
			MatterTaskPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			CancelMatterTaskPacket.class,
			CancelMatterTaskPacket::write,
			CancelMatterTaskPacket::read,
			CancelMatterTaskPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			PatternReplicationRequestPacket.class,
			PatternReplicationRequestPacket::write,
			PatternReplicationRequestPacket::read,
			PatternReplicationRequestPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			RegistryPacketClear.class,
			RegistryPacketClear::write,
			RegistryPacketClear.Companion::read,
			RegistryPacketClear::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			RegistryPacketUpdate.class,
			RegistryPacketUpdate::write,
			RegistryPacketUpdate.Companion::read,
			RegistryPacketUpdate::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			RegistryPacketFullUpdate.class,
			RegistryPacketFullUpdate::write,
			RegistryPacketFullUpdate.Companion::read,
			RegistryPacketFullUpdate::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			RegistryPacketRemove.class,
			RegistryPacketRemove::write,
			RegistryPacketRemove.Companion::read,
			RegistryPacketRemove::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidResearchRequestPacket.class,
			AndroidResearchRequestPacket::write,
			AndroidResearchRequestPacket::read,
			AndroidResearchRequestPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			AndroidResearchPacket.class,
			AndroidResearchPacket::write,
			AndroidResearchPacket::read,
			AndroidResearchPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			MatterBottlerSwitchPacket.class,
			MatterBottlerSwitchPacket::write,
			MatterBottlerSwitchPacket::read,
			MatterBottlerSwitchPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			ClearPacket.class,
			ClearPacket::write,
			ClearPacket::read,
			ClearPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			StackAddPacket.class,
			StackAddPacket::write,
			StackAddPacket::read,
			StackAddPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			StackChangePacket.class,
			StackChangePacket::write,
			StackChangePacket::read,
			StackChangePacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			StackRemovePacket.class,
			StackRemovePacket::write,
			StackRemovePacket::read,
			StackRemovePacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			InteractPacket.class,
			InteractPacket::write,
			InteractPacket::read,
			InteractPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			SetCarriedPacket.class,
			SetCarriedPacket::write,
			SetCarriedPacket::read,
			SetCarriedPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			MenuDriveViewer.FilterSwitchPacket.class,
			MenuDriveViewer.FilterSwitchPacket::write,
			MenuDriveViewer.FilterSwitchPacket::read,
			MenuDriveViewer.FilterSwitchPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			MenuDriveViewer.FilterSetPacket.class,
			MenuDriveViewer.FilterSetPacket::write,
			MenuDriveViewer.FilterSetPacket::read,
			MenuDriveViewer.FilterSetPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_SERVER)
		);

		CHANNEL.registerMessage(
			next_network_id++,
			EnergyCounterPacket.class,
			EnergyCounterPacket::write,
			EnergyCounterPacket.Companion::read,
			EnergyCounterPacket::play,
			Optional.of(NetworkDirection.PLAY_TO_CLIENT)
		);
	}
}
