package ru.dbotthepony.mc.otm.registry;

import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.tags.BlockTags;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.food.FoodProperties;
import net.minecraft.world.item.*;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraft.world.level.Level;
import net.minecraftforge.common.ForgeTier;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import org.jetbrains.annotations.Nullable;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.core.ImpreciseFraction;
import ru.dbotthepony.mc.otm.item.*;

import javax.annotation.Nonnull;
import java.util.ConcurrentModificationException;
import java.util.List;

public class MItems {
	static {
		if (OverdriveThatMatters.INSTANCE == null) {
			throw new ConcurrentModificationException("Accessing Registry before OverdriveThatMatters class is initialized. This is not supported! If no other mods are installed this is a bug.");
		}
	}

	private static final Item.Properties DEFAULT_PROPERTIES = new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB);

	public static final BlockItem ANDROID_STATION = new BlockItem(MBlocks.ANDROID_STATION, DEFAULT_PROPERTIES);
	public static final BlockItem BATTERY_BANK = new BlockItem(MBlocks.BATTERY_BANK, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_DECOMPOSER = new BlockItem(MBlocks.MATTER_DECOMPOSER, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_CAPACITOR_BANK = new BlockItem(MBlocks.MATTER_CAPACITOR_BANK, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_CABLE = new BlockItem(MBlocks.MATTER_CABLE, DEFAULT_PROPERTIES);
	public static final BlockItem PATTERN_STORAGE = new BlockItem(MBlocks.PATTERN_STORAGE, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_SCANNER = new BlockItem(MBlocks.MATTER_SCANNER, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_PANEL = new BlockItem(MBlocks.MATTER_PANEL, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_REPLICATOR = new BlockItem(MBlocks.MATTER_REPLICATOR, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_BOTTLER = new BlockItem(MBlocks.MATTER_BOTTLER, DEFAULT_PROPERTIES);
	public static final BlockItem DRIVE_VIEWER = new BlockItem(MBlocks.DRIVE_VIEWER, DEFAULT_PROPERTIES);
	public static final BlockItem CARGO_CRATE = new BlockItem(MBlocks.CARGO_CRATE, DEFAULT_PROPERTIES);
	public static final BlockItem TRITANIUM_ORE = new BlockItem(MBlocks.TRITANIUM_ORE, DEFAULT_PROPERTIES);
	public static final BlockItem DEEPSLATE_TRITANIUM_ORE = new BlockItem(MBlocks.DEEPSLATE_TRITANIUM_ORE, DEFAULT_PROPERTIES);
	public static final BlockItem TRITANIUM_RAW_BLOCK = new BlockItem(MBlocks.TRITANIUM_RAW_BLOCK, DEFAULT_PROPERTIES);
	public static final BlockItem DRIVE_RACK = new BlockItem(MBlocks.DRIVE_RACK, DEFAULT_PROPERTIES);
	public static final BlockItem ITEM_MONITOR = new BlockItem(MBlocks.ITEM_MONITOR, DEFAULT_PROPERTIES);
	public static final BlockItem ENERGY_COUNTER = new BlockItem(MBlocks.ENERGY_COUNTER, DEFAULT_PROPERTIES);
	public static final BlockItem CHEMICAL_GENERATOR = new BlockItem(MBlocks.CHEMICAL_GENERATOR, DEFAULT_PROPERTIES);
	public static final BlockItem PLATE_PRESS = new BlockItem(MBlocks.PLATE_PRESS, DEFAULT_PROPERTIES);
	public static final BlockItem MATTER_RECYCLER = new BlockItem(MBlocks.MATTER_RECYCLER, DEFAULT_PROPERTIES);
	public static final BlockItem GRAVITATION_STABILIZER = new BlockItem(MBlocks.GRAVITATION_STABILIZER, DEFAULT_PROPERTIES) {
		@Override
		public void appendHoverText(@Nonnull ItemStack p_40572_, @Nullable Level p_40573_, @Nonnull List<Component> p_40574_, @Nonnull TooltipFlag p_40575_) {
			super.appendHoverText(p_40572_, p_40573_, p_40574_, p_40575_);

			p_40574_.add(new TranslatableComponent("block.overdrive_that_matters.gravitation_stabilizer.desc").withStyle(ChatFormatting.GRAY));
			p_40574_.add(new TranslatableComponent("block.overdrive_that_matters.gravitation_stabilizer.desc2").withStyle(ChatFormatting.DARK_GRAY));
			p_40574_.add(new TranslatableComponent("block.overdrive_that_matters.gravitation_stabilizer.desc3").withStyle(ChatFormatting.DARK_GRAY));
			p_40574_.add(new TranslatableComponent("block.overdrive_that_matters.gravitation_stabilizer.desc4").withStyle(ChatFormatting.DARK_GRAY));
		}
	};

	public static final BlockItem DEBUG_EXPLOSION_SMALL = new BlockItem(MBlocks.DEBUG_EXPLOSION_SMALL, DEFAULT_PROPERTIES);
	public static final BlockItem DEBUG_SPHERE_POINTS = new BlockItem(MBlocks.DEBUG_SPHERE_POINTS, DEFAULT_PROPERTIES);

	public static final ItemMatterDust MATTER_DUST = new ItemMatterDust();

	public static final Item TRITANIUM_ORE_CLUMP = new Item(DEFAULT_PROPERTIES);
	public static final Item TRITANIUM_INGOT = new Item(DEFAULT_PROPERTIES);

	public static final ForgeTier TRITANIUM_COMPONENT;

	static {
		final var component = Ingredient.of(TRITANIUM_INGOT);

		TRITANIUM_COMPONENT = new ForgeTier(
			Tiers.IRON.getLevel(),
			3072,
			Tiers.IRON.getSpeed() * 1.1f,
			3.5f,
			16,
			BlockTags.NEEDS_IRON_TOOL,
			() -> component
		);
	}

	private static final Item.Properties TOOLS_PROPRTIES = new Item.Properties().tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB);
	public static final SwordItem TRITANIUM_SWORD = new SwordItem(TRITANIUM_COMPONENT, 4, -2.7f, TOOLS_PROPRTIES);
	public static final ShovelItem TRITANIUM_SHOVEL = new ShovelItem(TRITANIUM_COMPONENT, 1.5f, -2.4f, TOOLS_PROPRTIES);
	public static final AxeItem TRITANIUM_AXE = new AxeItem(TRITANIUM_COMPONENT, 8.5f, -3.4f, TOOLS_PROPRTIES);
	public static final PickaxeItem TRITANIUM_PICKAXE = new PickaxeItem(TRITANIUM_COMPONENT, 2, -2.8f, TOOLS_PROPRTIES);
	public static final HoeItem TRITANIUM_HOE = new HoeItem(TRITANIUM_COMPONENT, 0, -3.4f, TOOLS_PROPRTIES);

	public static final Item[] TRITANIUM_TOOLS = new Item[]{
		TRITANIUM_SWORD,
		TRITANIUM_SHOVEL,
		TRITANIUM_AXE,
		TRITANIUM_PICKAXE,
		TRITANIUM_HOE
	};

	public static final ItemTritaniumArmor TRITANIUM_HELMET = new ItemTritaniumArmor(EquipmentSlot.HEAD);
	public static final ItemTritaniumArmor TRITANIUM_CHESTPLATE = new ItemTritaniumArmor(EquipmentSlot.CHEST);
	public static final ItemTritaniumArmor TRITANIUM_PANTS = new ItemTritaniumArmor(EquipmentSlot.LEGS);
	public static final ItemTritaniumArmor TRITANIUM_BOOTS = new ItemTritaniumArmor(EquipmentSlot.FEET);

	public static final Item[] TRITANIUM_ARMOR = new Item[]{
		TRITANIUM_HELMET,
		TRITANIUM_CHESTPLATE,
		TRITANIUM_PANTS,
		TRITANIUM_BOOTS
	};

	public static final Item MATTER_IO_PORT = new Item(DEFAULT_PROPERTIES);
	public static final Item MATTER_TRANSFORM_MATRIX = new Item(DEFAULT_PROPERTIES);
	public static final Item ENERGY_BUS = new Item(DEFAULT_PROPERTIES);
	public static final Item ELECTRIC_PARTS = new Item(DEFAULT_PROPERTIES);
	public static final Item MACHINE_FRAME = new Item(DEFAULT_PROPERTIES);
	public static final Item TRITANIUM_PLATE = new Item(DEFAULT_PROPERTIES);
	public static final Item IRON_PLATE = new Item(DEFAULT_PROPERTIES);
	public static final Item COPPER_WIRING = new Item(DEFAULT_PROPERTIES);
	public static final Item GOLD_WIRING = new Item(DEFAULT_PROPERTIES);
	public static final Item PORTABLE_CONDENSATION_DRIVE_CASING = new Item(DEFAULT_PROPERTIES);
	public static final Item PORTABLE_DENSE_CONDENSATION_DRIVE_CASING = new Item(DEFAULT_PROPERTIES);
	public static final Item CIRCUIT_PLATING = new Item(DEFAULT_PROPERTIES);
	public static final Item BASIC_CONTROL_CIRCUIT = new Item(DEFAULT_PROPERTIES);
	public static final Item ADVANCED_CONTROL_CIRCUIT = new Item(DEFAULT_PROPERTIES);

	public static final Item BLACK_HOLE_SCANNER = new Item(DEFAULT_PROPERTIES) {
		@Override
		public void appendHoverText(@Nonnull ItemStack p_40572_, @Nullable Level p_40573_, @Nonnull List<Component> p_40574_, @Nonnull TooltipFlag p_40575_) {
			super.appendHoverText(p_40572_, p_40573_, p_40574_, p_40575_);

			p_40574_.add(new TranslatableComponent("item.overdrive_that_matters.black_hole_scanner.desc").withStyle(ChatFormatting.GRAY));
			p_40574_.add(new TranslatableComponent("item.overdrive_that_matters.black_hole_scanner.desc2").withStyle(ChatFormatting.DARK_GRAY));
		}
	};
	public static final Item GRAVITATION_FIELD_LIMITER = new Item(DEFAULT_PROPERTIES);
	public static final Item GRAVITATION_FIELD_SENSOR = new Item(DEFAULT_PROPERTIES);
	public static final ItemPortableGravitationStabilizer PORTABLE_GRAVITATION_STABILIZER = new ItemPortableGravitationStabilizer();

	public static final BlockItem BLACK_HOLE = new BlockItem(MBlocks.BLACK_HOLE, DEFAULT_PROPERTIES);

	public static final ItemGravitationalDisruptor GRAVITATIONAL_DISRUPTOR = new ItemGravitationalDisruptor();

	public static final ItemPill PILL_ANDROID = new ItemPill(PillType.BECOME_ANDROID);
	public static final ItemPill PILL_HUMANE = new ItemPill(PillType.BECOME_HUMANE);
	public static final ItemPill PILL_OBLIVION = new ItemPill(PillType.OBLIVION);
	public static final ItemPillHeal PILL_HEAL = new ItemPillHeal();

	public static final ItemBattery BATTERY_CRUDE = new ItemBattery(new ImpreciseFraction(30_000), new ImpreciseFraction(150), new ImpreciseFraction(150));
	public static final ItemBattery BATTERY_BASIC = new ItemBattery(new ImpreciseFraction(60_000), new ImpreciseFraction(300), new ImpreciseFraction(300));
	public static final ItemBattery BATTERY_NORMAL = new ItemBattery(new ImpreciseFraction(250_000), new ImpreciseFraction(1000), new ImpreciseFraction(1000));
	public static final ItemBattery BATTERY_DENSE = new ItemBattery(new ImpreciseFraction(1_000_000), new ImpreciseFraction(2000), new ImpreciseFraction(2000));
	public static final ItemBattery BATTERY_CAPACITOR = new ItemBattery(new ImpreciseFraction(150_000), new ImpreciseFraction(15000), new ImpreciseFraction(15000));
	public static final ItemBattery BATTERY_CREATIVE = new ItemBattery();

	public static final ItemBattery[] BATTERIES = {
		BATTERY_CRUDE,
		BATTERY_BASIC,
		BATTERY_NORMAL,
		BATTERY_DENSE,
		BATTERY_CAPACITOR,
	};

	public static final Item MATTER_CAPACITOR_PARTS = new Item(new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB));

	public static final Item[] DATAGEN_COMPONENTS = {
		ENERGY_BUS,
		ELECTRIC_PARTS,
		TRITANIUM_PLATE,
		IRON_PLATE,
		COPPER_WIRING,
		GOLD_WIRING,
		CIRCUIT_PLATING,
		BASIC_CONTROL_CIRCUIT,
		ADVANCED_CONTROL_CIRCUIT,
		MATTER_CAPACITOR_PARTS,
		MATTER_IO_PORT,
		MATTER_TRANSFORM_MATRIX,
	};

	public static final ItemMatterCapacitor MATTER_CAPACITOR_BASIC = new ItemMatterCapacitor(new ImpreciseFraction("4"));
	public static final ItemMatterCapacitor MATTER_CAPACITOR_NORMAL = new ItemMatterCapacitor(new ImpreciseFraction("10"));
	public static final ItemMatterCapacitor MATTER_CAPACITOR_DENSE = new ItemMatterCapacitor(new ImpreciseFraction("40"));
	public static final ItemMatterCapacitor MATTER_CAPACITOR_CREATIVE = new ItemMatterCapacitor();

	public static final ItemPatternStorage PATTERN_DRIVE_NORMAL = new ItemPatternStorage(4);
	public static final ItemPatternStorage PATTERN_DRIVE_CREATIVE = new ItemPatternStorage();

	public static final ItemPortableCondensationDrive PORTABLE_CONDENSATION_DRIVE = new ItemPortableCondensationDrive(4000);
	public static final ItemPortableCondensationDrive PORTABLE_DENSE_CONDENSATION_DRIVE = new ItemPortableCondensationDrive(25000);

	public static final Item NUTRIENT_PASTE = new Item(new Item.Properties().stacksTo(64).food(new FoodProperties.Builder().meat().nutrition(8).saturationMod(0.9F).build()).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB));

	public static final Item[] CRATES = new Item[Registry.CRATES.length];

	public static final Item TRITANIUM_BLOCK = new BlockItem(MBlocks.TRITANIUM_BLOCK, DEFAULT_PROPERTIES);
	public static final Item TRITANIUM_STRIPED_BLOCK = new BlockItem(MBlocks.TRITANIUM_STRIPED_BLOCK, DEFAULT_PROPERTIES);
	public static final Item CARBON_FIBRE_BLOCK = new BlockItem(MBlocks.CARBON_FIBRE_BLOCK, DEFAULT_PROPERTIES);

	static {
		for (int i = 0; i < Registry.CRATES.length; i++) {
			CRATES[i] = new BlockItem(MBlocks.CRATES[i], new Item.Properties().stacksTo(64).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB));
			CRATES[i].setRegistryName(Registry.CRATES[i].name());
		}

		ANDROID_STATION.setRegistryName(MNames.ANDROID_STATION);
		BATTERY_BANK.setRegistryName(MNames.BATTERY_BANK);
		MATTER_DECOMPOSER.setRegistryName(MNames.MATTER_DECOMPOSER);
		MATTER_CAPACITOR_BANK.setRegistryName(MNames.MATTER_CAPACITOR_BANK);
		MATTER_CABLE.setRegistryName(MNames.MATTER_CABLE);
		PATTERN_STORAGE.setRegistryName(MNames.PATTERN_STORAGE);
		MATTER_SCANNER.setRegistryName(MNames.MATTER_SCANNER);
		MATTER_PANEL.setRegistryName(MNames.MATTER_PANEL);
		MATTER_REPLICATOR.setRegistryName(MNames.MATTER_REPLICATOR);
		MATTER_BOTTLER.setRegistryName(MNames.MATTER_BOTTLER);
		DRIVE_VIEWER.setRegistryName(MNames.DRIVE_VIEWER);
		BLACK_HOLE.setRegistryName(MNames.BLACK_HOLE);
		CARGO_CRATE.setRegistryName(MNames.CARGO_CRATE);
		DRIVE_RACK.setRegistryName(MNames.DRIVE_RACK);
		ITEM_MONITOR.setRegistryName(MNames.ITEM_MONITOR);
		ENERGY_COUNTER.setRegistryName(MNames.ENERGY_COUNTER);
		CHEMICAL_GENERATOR.setRegistryName(MNames.CHEMICAL_GENERATOR);
		PLATE_PRESS.setRegistryName(MNames.PLATE_PRESS);
		GRAVITATION_STABILIZER.setRegistryName(MNames.GRAVITATION_STABILIZER);
		MATTER_RECYCLER.setRegistryName(MNames.MATTER_RECYCLER);
		MATTER_DUST.setRegistryName(MNames.MATTER_DUST);

		DEBUG_EXPLOSION_SMALL.setRegistryName(MNames.DEBUG_EXPLOSION_SMALL);
		DEBUG_SPHERE_POINTS.setRegistryName(MNames.DEBUG_SPHERE_POINTS);

		TRITANIUM_ORE.setRegistryName(MNames.TRITANIUM_ORE);
		DEEPSLATE_TRITANIUM_ORE.setRegistryName(MNames.DEEPSLATE_TRITANIUM_ORE);
		TRITANIUM_RAW_BLOCK.setRegistryName(MNames.TRITANIUM_RAW_BLOCK);
		TRITANIUM_ORE_CLUMP.setRegistryName(MNames.TRITANIUM_ORE_CLUMP);
		TRITANIUM_INGOT.setRegistryName(MNames.TRITANIUM_INGOT);
		MATTER_IO_PORT.setRegistryName(MNames.MATTER_IO_PORT);
		MATTER_TRANSFORM_MATRIX.setRegistryName(MNames.MATTER_TRANSFORM_MATRIX);
		ENERGY_BUS.setRegistryName(MNames.ENERGY_BUS);
		ELECTRIC_PARTS.setRegistryName(MNames.ELECTRIC_PARTS);
		MACHINE_FRAME.setRegistryName(MNames.MACHINE_FRAME);
		TRITANIUM_PLATE.setRegistryName(MNames.TRITANIUM_PLATE);
		IRON_PLATE.setRegistryName(MNames.IRON_PLATE);
		COPPER_WIRING.setRegistryName(MNames.COPPER_WIRING);
		GOLD_WIRING.setRegistryName(MNames.GOLD_WIRING);
		PORTABLE_CONDENSATION_DRIVE_CASING.setRegistryName(MNames.PORTABLE_CONDENSATION_DRIVE_CASING);
		PORTABLE_DENSE_CONDENSATION_DRIVE_CASING.setRegistryName(MNames.PORTABLE_DENSE_CONDENSATION_DRIVE_CASING);
		CIRCUIT_PLATING.setRegistryName(MNames.CIRCUIT_PLATING);
		BASIC_CONTROL_CIRCUIT.setRegistryName(MNames.BASIC_CONTROL_CIRCUIT);
		ADVANCED_CONTROL_CIRCUIT.setRegistryName(MNames.ADVANCED_CONTROL_CIRCUIT);

		BLACK_HOLE_SCANNER.setRegistryName(MNames.BLACK_HOLE_SCANNER);
		GRAVITATION_FIELD_LIMITER.setRegistryName(MNames.GRAVITATION_FIELD_LIMITER);
		GRAVITATION_FIELD_SENSOR.setRegistryName(MNames.GRAVITATION_FIELD_SENSOR);
		PORTABLE_GRAVITATION_STABILIZER.setRegistryName(MNames.PORTABLE_GRAVITATION_STABILIZER);

		GRAVITATIONAL_DISRUPTOR.setRegistryName(MNames.GRAVITATIONAL_DISRUPTOR);
		PILL_ANDROID.setRegistryName(MNames.PILL_ANDROID);
		PILL_HUMANE.setRegistryName(MNames.PILL_HUMANE);
		PILL_OBLIVION.setRegistryName(MNames.PILL_OBLIVION);
		PILL_HEAL.setRegistryName(MNames.PILL_HEAL);
		BATTERY_CRUDE.setRegistryName(MNames.BATTERY_CRUDE);
		BATTERY_BASIC.setRegistryName(MNames.BATTERY_BASIC);
		BATTERY_NORMAL.setRegistryName(MNames.BATTERY_NORMAL);
		BATTERY_DENSE.setRegistryName(MNames.BATTERY_DENSE);
		BATTERY_CAPACITOR.setRegistryName(MNames.BATTERY_CAPACITOR);
		BATTERY_CREATIVE.setRegistryName(MNames.BATTERY_CREATIVE);

		MATTER_CAPACITOR_PARTS.setRegistryName(MNames.MATTER_CAPACITOR_PARTS);
		MATTER_CAPACITOR_BASIC.setRegistryName(MNames.MATTER_CAPACITOR_BASIC);
		MATTER_CAPACITOR_NORMAL.setRegistryName(MNames.MATTER_CAPACITOR_NORMAL);
		MATTER_CAPACITOR_DENSE.setRegistryName(MNames.MATTER_CAPACITOR_DENSE);
		MATTER_CAPACITOR_CREATIVE.setRegistryName(MNames.MATTER_CAPACITOR_CREATIVE);

		PATTERN_DRIVE_NORMAL.setRegistryName(MNames.PATTERN_DRIVE_NORMAL);
		PATTERN_DRIVE_CREATIVE.setRegistryName(MNames.PATTERN_DRIVE_CREATIVE);
		PORTABLE_CONDENSATION_DRIVE.setRegistryName(MNames.PORTABLE_CONDENSATION_DRIVE);
		PORTABLE_DENSE_CONDENSATION_DRIVE.setRegistryName(MNames.PORTABLE_DENSE_CONDENSATION_DRIVE);

		NUTRIENT_PASTE.setRegistryName(MNames.NUTRIENT_PASTE);

		TRITANIUM_BLOCK.setRegistryName(MNames.TRITANIUM_BLOCK);
		TRITANIUM_STRIPED_BLOCK.setRegistryName(MNames.TRITANIUM_STRIPED_BLOCK);
		CARBON_FIBRE_BLOCK.setRegistryName(MNames.CARBON_FIBRE_BLOCK);

		TRITANIUM_SWORD.setRegistryName(MNames.TRITANIUM_SWORD);
		TRITANIUM_PICKAXE.setRegistryName(MNames.TRITANIUM_PICKAXE);
		TRITANIUM_SHOVEL.setRegistryName(MNames.TRITANIUM_SHOVEL);
		TRITANIUM_AXE.setRegistryName(MNames.TRITANIUM_AXE);
		TRITANIUM_HOE.setRegistryName(MNames.TRITANIUM_HOE);

		TRITANIUM_HELMET.setRegistryName(MNames.TRITANIUM_HELMET);
		TRITANIUM_CHESTPLATE.setRegistryName(MNames.TRITANIUM_CHESTPLATE);
		TRITANIUM_PANTS.setRegistryName(MNames.TRITANIUM_PANTS);
		TRITANIUM_BOOTS.setRegistryName(MNames.TRITANIUM_BOOTS);
	}

	@SubscribeEvent
	@SuppressWarnings("unused")
	public static void register(final RegistryEvent.Register<Item> event) {
		event.getRegistry().register(ANDROID_STATION);
		event.getRegistry().register(BATTERY_BANK);
		event.getRegistry().register(MATTER_DECOMPOSER);
		event.getRegistry().register(MATTER_CAPACITOR_BANK);
		event.getRegistry().register(MATTER_CABLE);
		event.getRegistry().register(PATTERN_STORAGE);
		event.getRegistry().register(MATTER_SCANNER);
		event.getRegistry().register(MATTER_PANEL);
		event.getRegistry().register(MATTER_REPLICATOR);
		event.getRegistry().register(MATTER_BOTTLER);
		event.getRegistry().register(DRIVE_VIEWER);
		event.getRegistry().register(BLACK_HOLE);
		event.getRegistry().register(CARGO_CRATE);
		event.getRegistry().register(DRIVE_RACK);
		event.getRegistry().register(ITEM_MONITOR);
		event.getRegistry().register(ENERGY_COUNTER);
		event.getRegistry().register(CHEMICAL_GENERATOR);
		event.getRegistry().register(PLATE_PRESS);
		event.getRegistry().register(GRAVITATION_STABILIZER);
		event.getRegistry().register(MATTER_DUST);
		event.getRegistry().register(MATTER_RECYCLER);

		event.getRegistry().register(DEBUG_EXPLOSION_SMALL);
		event.getRegistry().register(DEBUG_SPHERE_POINTS);

		event.getRegistry().register(TRITANIUM_ORE);
		event.getRegistry().register(DEEPSLATE_TRITANIUM_ORE);
		event.getRegistry().register(TRITANIUM_RAW_BLOCK);

		event.getRegistry().register(TRITANIUM_ORE_CLUMP);
		event.getRegistry().register(TRITANIUM_INGOT);
		event.getRegistry().register(MATTER_IO_PORT);
		event.getRegistry().register(MATTER_TRANSFORM_MATRIX);
		event.getRegistry().register(ENERGY_BUS);
		event.getRegistry().register(ELECTRIC_PARTS);
		event.getRegistry().register(MACHINE_FRAME);
		event.getRegistry().register(TRITANIUM_PLATE);
		event.getRegistry().register(IRON_PLATE);
		event.getRegistry().register(COPPER_WIRING);
		event.getRegistry().register(GOLD_WIRING);
		event.getRegistry().register(PORTABLE_CONDENSATION_DRIVE_CASING);
		event.getRegistry().register(PORTABLE_DENSE_CONDENSATION_DRIVE_CASING);
		event.getRegistry().register(CIRCUIT_PLATING);
		event.getRegistry().register(BASIC_CONTROL_CIRCUIT);
		event.getRegistry().register(ADVANCED_CONTROL_CIRCUIT);

		event.getRegistry().register(BLACK_HOLE_SCANNER);
		event.getRegistry().register(GRAVITATION_FIELD_LIMITER);
		event.getRegistry().register(GRAVITATION_FIELD_SENSOR);
		event.getRegistry().register(PORTABLE_GRAVITATION_STABILIZER);

		event.getRegistry().register(GRAVITATIONAL_DISRUPTOR);
		event.getRegistry().register(PILL_ANDROID);
		event.getRegistry().register(PILL_HUMANE);
		event.getRegistry().register(PILL_OBLIVION);
		event.getRegistry().register(PILL_HEAL);
		event.getRegistry().register(BATTERY_CRUDE);
		event.getRegistry().register(BATTERY_BASIC);
		event.getRegistry().register(BATTERY_NORMAL);
		event.getRegistry().register(BATTERY_DENSE);
		event.getRegistry().register(BATTERY_CAPACITOR);
		event.getRegistry().register(BATTERY_CREATIVE);

		event.getRegistry().register(MATTER_CAPACITOR_PARTS);
		event.getRegistry().register(MATTER_CAPACITOR_BASIC);
		event.getRegistry().register(MATTER_CAPACITOR_NORMAL);
		event.getRegistry().register(MATTER_CAPACITOR_DENSE);
		event.getRegistry().register(MATTER_CAPACITOR_CREATIVE);

		event.getRegistry().register(PATTERN_DRIVE_NORMAL);
		event.getRegistry().register(PATTERN_DRIVE_CREATIVE);
		event.getRegistry().register(PORTABLE_CONDENSATION_DRIVE);
		event.getRegistry().register(PORTABLE_DENSE_CONDENSATION_DRIVE);

		event.getRegistry().register(NUTRIENT_PASTE);

		event.getRegistry().register(TRITANIUM_BLOCK);
		event.getRegistry().register(TRITANIUM_STRIPED_BLOCK);
		event.getRegistry().register(CARBON_FIBRE_BLOCK);

		for (var crate : CRATES) {
			event.getRegistry().register(crate);
		}

		for (var glass : Registry.INDUSTRIAL_GLASS_LIST) {
			event.getRegistry().register(glass.item());
			event.getRegistry().register(glass.paneItem());
		}

		event.getRegistry().register(TRITANIUM_SWORD);
		event.getRegistry().register(TRITANIUM_SHOVEL);
		event.getRegistry().register(TRITANIUM_AXE);
		event.getRegistry().register(TRITANIUM_PICKAXE);
		event.getRegistry().register(TRITANIUM_HOE);

		event.getRegistry().register(TRITANIUM_HELMET);
		event.getRegistry().register(TRITANIUM_CHESTPLATE);
		event.getRegistry().register(TRITANIUM_PANTS);
		event.getRegistry().register(TRITANIUM_BOOTS);

		// OverdriveThatMatters.LOGGER.info("Registered items");
	}
}
