package ru.dbotthepony.mc.otm.registry;

import net.minecraft.client.gui.screens.MenuScreens;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.stats.StatFormatter;
import net.minecraft.tags.ItemTags;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.item.*;
import net.minecraft.world.item.crafting.Recipe;
import net.minecraft.world.item.crafting.RecipeSerializer;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.level.block.*;
import net.minecraft.world.level.block.state.BlockBehaviour;
import net.minecraft.world.level.material.Material;
import net.minecraft.world.level.material.MaterialColor;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.registries.ForgeRegistry;
import net.minecraftforge.registries.RegistryBuilder;
import org.jetbrains.annotations.Nullable;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.android.AndroidResearchBuilder;
import ru.dbotthepony.mc.otm.android.AndroidResearchType;
import ru.dbotthepony.mc.otm.android.feature.AndroidNanobotsArmor;
import ru.dbotthepony.mc.otm.android.AndroidFeatureType;
import ru.dbotthepony.mc.otm.client.render.SkinElement;
import ru.dbotthepony.mc.otm.menu.*;
import ru.dbotthepony.mc.otm.client.screen.*;
import ru.dbotthepony.mc.otm.recipe.PlatePressRecipe;
import ru.dbotthepony.mc.otm.recipe.PlatePressRecipeFactory;

import net.minecraftforge.common.Tags.IOptionalNamedTag;
import ru.dbotthepony.mc.otm.registry.AndroidFeatures;
import ru.dbotthepony.mc.otm.registry.MItems;
import ru.dbotthepony.mc.otm.registry.MNames;

import java.util.ConcurrentModificationException;
import java.util.Set;

public class Registry {
	static {
		if (OverdriveThatMatters.INSTANCE == null) {
			throw new ConcurrentModificationException("Accessing Registry before OverdriveThatMatters class is initialized. This is not supported! If no other mods are installed this is a bug.");
		}
	}

	public static final DamageSource DAMAGE_BECOME_ANDROID = new DamageSource("otm_become_android");
	public static final DamageSource DAMAGE_BECOME_HUMANE = new DamageSource("otm_become_humane");
	public static final DamageSource DAMAGE_EVENT_HORIZON = new DamageSource("otm_event_horizon");
	public static final DamageSource DAMAGE_HAWKING_RADIATION = new DamageSource("otm_hawking_radiation");

	static {
		DAMAGE_BECOME_ANDROID.bypassArmor().bypassInvul().bypassMagic();
		DAMAGE_BECOME_HUMANE.bypassArmor().bypassInvul().bypassMagic();
		DAMAGE_EVENT_HORIZON.bypassMagic().bypassArmor();
		// DAMAGE_HAWKING_RADIATION.bypassMagic().bypassArmor();
	}

	public static final ForgeRegistry<AndroidFeatureType<?>> ANDROID_FEATURES;
	public static final ForgeRegistry<AndroidResearchType<?>> ANDROID_RESEARCH;

	static {
		var builder = new RegistryBuilder<AndroidFeatureType<?>>();
		builder.setName(new ResourceLocation(OverdriveThatMatters.MOD_ID, "android_features"));

		// make it shut up
		builder.setType(c(AndroidFeatureType.class));
		ANDROID_FEATURES = (ForgeRegistry<AndroidFeatureType<?>>) builder.create();

		var builder2 = new RegistryBuilder<AndroidResearchType<?>>();
		builder2.setName(new ResourceLocation(OverdriveThatMatters.MOD_ID, "android_research"));

		// make it shut up
		builder2.setType(c(AndroidResearchType.class));
		ANDROID_RESEARCH = (ForgeRegistry<AndroidResearchType<?>>) builder2.create();
	}

	public static void dummy() {}

	@SuppressWarnings("unchecked")
	private static <T> Class<T> c(Class<?> cls) { return (Class<T>) cls; }

	record CrateProps(MaterialColor color, ResourceLocation name) {
		public CrateProps(MaterialColor color, String name) {
			this(color, new ResourceLocation(OverdriveThatMatters.MOD_ID, "crate_" + name));
		}
	}

	public static final CrateProps[] CRATES = new CrateProps[] {
		new CrateProps(MaterialColor.COLOR_RED, "red"),
		new CrateProps(MaterialColor.COLOR_BLUE, "blue"),
		new CrateProps(MaterialColor.COLOR_YELLOW, "yellow"),
		new CrateProps(MaterialColor.COLOR_GREEN, "green"),
		new CrateProps(MaterialColor.COLOR_BLACK, "black"),
		new CrateProps(MaterialColor.COLOR_PINK, "pink"),
		new CrateProps(MaterialColor.COLOR_PURPLE, "purple"),
	};

	public record IndustrialGlassProps(@Nullable DyeColor color, ResourceLocation name, ResourceLocation namePane, Block block, Block pane, BlockItem item, BlockItem paneItem) {
		public IndustrialGlassProps(@Nullable DyeColor color, ResourceLocation name, ResourceLocation namePane, Block block, Block pane, BlockItem item, BlockItem paneItem) {
			this.color = color;
			this.name = name;
			this.namePane = namePane;
			this.block = block;
			this.pane = pane;
			this.item = item;
			this.paneItem = paneItem;

			this.block.setRegistryName(this.name);
			this.pane.setRegistryName(this.namePane);
			this.item.setRegistryName(this.name);
			this.paneItem.setRegistryName(this.namePane);
		}

		IndustrialGlassProps(@Nullable DyeColor color, ResourceLocation name, ResourceLocation namePane, Block block, Block pane) {
			this(color, name, namePane, block, pane,
				new BlockItem(block, new Item.Properties().tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB).stacksTo(64)),
				new BlockItem(pane, new Item.Properties().tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB).stacksTo(64)));
		}

		IndustrialGlassProps(@Nullable DyeColor color, ResourceLocation name, ResourceLocation namePane) {
			this(color, name, namePane,
				glass(color, BlockBehaviour.Properties.of(Material.GLASS, color != null ? color.getMaterialColor() : MaterialColor.NONE)
					.strength(1.5F, 5.0F)
					.requiresCorrectToolForDrops()
					.sound(SoundType.GLASS)
					.noOcclusion()
					.isValidSpawn((_a, _b, _c, _d) -> false)
					.isRedstoneConductor((_a, _b, _c) -> false)
					.isSuffocating((_a, _b, _c) -> false)
					.isViewBlocking((_a, _b, _c) -> false)),
				pane(color, BlockBehaviour.Properties.of(Material.GLASS, color != null ? color.getMaterialColor() : MaterialColor.NONE)
					.strength(1.25F, 5.0F)
					.requiresCorrectToolForDrops()
					.sound(SoundType.GLASS)
					.noOcclusion())
			);
		}

		private static Block glass(@Nullable DyeColor color, BlockBehaviour.Properties props) {
			if (color == null) {
				return new GlassBlock(props);
			}

			return new StainedGlassBlock(color, props);
		}

		private static Block pane(@Nullable DyeColor color, BlockBehaviour.Properties props) {
			if (color == null) {
				return new IronBarsBlock(props);
			}

			return new StainedGlassPaneBlock(color, props);
		}

		IndustrialGlassProps(@Nullable DyeColor color) {
			this(color,
				new ResourceLocation(OverdriveThatMatters.MOD_ID, color != null ? "industrial_glass_" + color.getName() : "industrial_glass"),
				new ResourceLocation(OverdriveThatMatters.MOD_ID, color != null ? "industrial_glass_pane_" + color.getName() : "industrial_glass_pane")
			);
		}
	}

	public static final IndustrialGlassProps INDUSTRIAL_GLASS = new IndustrialGlassProps(null);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_WHITE = new IndustrialGlassProps(DyeColor.WHITE);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_ORANGE = new IndustrialGlassProps(DyeColor.ORANGE);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_MAGENTA = new IndustrialGlassProps(DyeColor.MAGENTA);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_LIGHT_BLUE = new IndustrialGlassProps(DyeColor.LIGHT_BLUE);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_YELLOW = new IndustrialGlassProps(DyeColor.YELLOW);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_LIME = new IndustrialGlassProps(DyeColor.LIME);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_PINK = new IndustrialGlassProps(DyeColor.PINK);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_GRAY = new IndustrialGlassProps(DyeColor.GRAY);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_LIGHT_GRAY = new IndustrialGlassProps(DyeColor.LIGHT_GRAY);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_CYAN = new IndustrialGlassProps(DyeColor.CYAN);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_PURPLE = new IndustrialGlassProps(DyeColor.PURPLE);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_BLUE = new IndustrialGlassProps(DyeColor.BLUE);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_BROWN = new IndustrialGlassProps(DyeColor.BROWN);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_GREEN = new IndustrialGlassProps(DyeColor.GREEN);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_RED = new IndustrialGlassProps(DyeColor.RED);
	public static final IndustrialGlassProps INDUSTRIAL_GLASS_BLACK = new IndustrialGlassProps(DyeColor.BLACK);

	public static final IndustrialGlassProps[] INDUSTRIAL_GLASS_LIST = new IndustrialGlassProps[] {
		INDUSTRIAL_GLASS,
		INDUSTRIAL_GLASS_WHITE,
		INDUSTRIAL_GLASS_ORANGE,
		INDUSTRIAL_GLASS_MAGENTA,
		INDUSTRIAL_GLASS_LIGHT_BLUE,
		INDUSTRIAL_GLASS_YELLOW,
		INDUSTRIAL_GLASS_LIME,
		INDUSTRIAL_GLASS_PINK,
		INDUSTRIAL_GLASS_GRAY,
		INDUSTRIAL_GLASS_LIGHT_GRAY,
		INDUSTRIAL_GLASS_CYAN,
		INDUSTRIAL_GLASS_PURPLE,
		INDUSTRIAL_GLASS_BLUE,
		INDUSTRIAL_GLASS_BROWN,
		INDUSTRIAL_GLASS_GREEN,
		INDUSTRIAL_GLASS_RED,
		INDUSTRIAL_GLASS_BLACK,
	};

}
