package ru.dbotthepony.mc.otm

import com.google.gson.JsonArray
import com.google.gson.JsonElement
import com.google.gson.JsonObject
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.core.Vec3i
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.ListTag
import net.minecraft.nbt.Tag
import net.minecraft.world.Container
import net.minecraft.world.item.ItemStack
import net.minecraft.world.phys.Vec3
import net.minecraftforge.common.util.LazyOptional
import java.util.function.Consumer

operator fun Direction.unaryMinus(): Direction = this.opposite
operator fun Vec3i.unaryMinus(): Vec3i = Vec3i(-x, -y, -z)
operator fun BlockPos.unaryMinus(): BlockPos = BlockPos(-x, -y, -z)

operator fun CompoundTag.set(index: String, value: Tag) = 				put(index, value)
operator fun CompoundTag.set(index: String, value: Int) = 				putInt(index, value)
operator fun CompoundTag.set(index: String, value: Byte) = 				putByte(index, value)
operator fun CompoundTag.set(index: String, value: Short) = 			putShort(index, value)
operator fun CompoundTag.set(index: String, value: Long) = 				putLong(index, value)
operator fun CompoundTag.set(index: String, value: Float) = 			putFloat(index, value)
operator fun CompoundTag.set(index: String, value: Double) = 			putDouble(index, value)
operator fun CompoundTag.set(index: String, value: String) = 			putString(index, value)
operator fun CompoundTag.set(index: String, value: Boolean) = 			putBoolean(index, value)
operator fun CompoundTag.set(index: String, value: ByteArray) = 		putByteArray(index, value)
operator fun CompoundTag.set(index: String, value: IntArray) = 			putIntArray(index, value)
operator fun CompoundTag.set(index: String, value: LongArray) = 		putLongArray(index, value)

operator fun Container.set(index: Int, value: ItemStack) = setItem(index, value)
operator fun Container.get(index: Int): ItemStack = getItem(index)

operator fun JsonObject.set(s: String, value: JsonElement) = add(s, value)

inline fun CompoundTag.ifHas(s: String, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null) {
		consumer(tag)
	}
}

inline fun CompoundTag.ifHas(s: String, type: Byte, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null && tag.id == type) {
		consumer(tag)
	}
}

inline fun <reified T : Tag> CompoundTag.ifHas(s: String, type: Class<T>, consumer: (T) -> Unit) {
	val tag = get(s)

	if (tag != null && tag::class.java === type) {
		consumer(tag as T)
	}
}

fun <T> LazyOptional<T>.orNull(): T? {
	if (!isPresent) {
		return null
	}

	return resolve().orElse(null)
}
