package ru.dbotthepony.mc.otm

import net.minecraft.client.multiplayer.ClientLevel
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.level.Level
import net.minecraftforge.event.TickEvent
import net.minecraftforge.event.TickEvent.WorldTickEvent
import net.minecraftforge.event.server.ServerAboutToStartEvent
import net.minecraftforge.event.server.ServerStoppingEvent
import net.minecraftforge.eventbus.api.EventPriority
import net.minecraftforge.eventbus.api.SubscribeEvent
import net.minecraftforge.fml.LogicalSide
import java.util.*
import java.util.function.Consumer
import java.util.function.Supplier

private val tick_until = WeakHashMap<Level, ArrayList<Supplier<Boolean>>>()
private val tick_once = WeakHashMap<Level, ArrayList<Runnable>>()

@SubscribeEvent(priority = EventPriority.HIGHEST)
@Suppress("unused")
fun onServerStarting(event: ServerAboutToStartEvent?) {
	tick_until.clear()
}

@SubscribeEvent(priority = EventPriority.HIGHEST)
@Suppress("unused")
fun onServerStopping(event: ServerStoppingEvent?) {
	tick_until.clear()
}

@SubscribeEvent(priority = EventPriority.LOWEST)
@Suppress("unused")
fun onPreTick(event: WorldTickEvent) {
	if (event.phase != TickEvent.Phase.START || event.side != LogicalSide.SERVER) return

	// удаляем список сразу что бы если кто-либо добавит туда элементы у нас была "копия"
	val until = tick_until.remove(event.world)

	if (until != null) {
		for (i in until.indices.reversed()) {
			if (until[i].get()) {
				until.removeAt(i)
			}
		}

		if (until.size != 0) {
			val replaced = tick_until.put(event.world, until)

			if (replaced != null) {
				until.addAll(replaced)
			}
		}
	}

	val once = tick_once.remove(event.world)

	if (once != null)
		for (ticker in once)
			ticker.run()
}

fun tickUntil(level: Level, ticker: Supplier<Boolean>) {
	tick_until.computeIfAbsent(level) { ArrayList() }.add(ticker)
}

fun tickUntilServer(level: Level, ticker: Supplier<Boolean>) {
	if (level is ServerLevel)
		tick_until.computeIfAbsent(level) { ArrayList() }.add(ticker)
}

fun tickUntilClient(level: Level, ticker: Supplier<Boolean>) {
	if (level is ClientLevel)
		tick_until.computeIfAbsent(level) { ArrayList() }.add(ticker)
}

fun tickOnce(level: Level, ticker: Runnable) {
	tick_once.computeIfAbsent(level) { ArrayList() }.add(ticker)
}

fun tickOnceServer(level: Level, ticker: Runnable) {
	if (level is ServerLevel)
		tick_once.computeIfAbsent(level) { ArrayList() }.add(ticker)
}

fun tickOnceClient(level: Level, ticker: Runnable) {
	if (level is ClientLevel)
		tick_once.computeIfAbsent(level) { ArrayList() }.add(ticker)
}
