package ru.dbotthepony.mc.otm.block

import net.minecraft.MethodsReturnNonnullByDefault
import javax.annotation.ParametersAreNonnullByDefault
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.properties.BooleanProperty
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import ru.dbotthepony.mc.otm.block.entity.BlockEntityBatteryBank
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraft.world.phys.shapes.VoxelShape
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.shapes.BlockShapes

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
class BlockBatteryBank : BlockMatteryRotatable(), EntityBlock {
	override fun getStateForPlacement(context: BlockPlaceContext): BlockState {
		var state = super.getStateForPlacement(context)!!

		for (prop in BATTERY_SLOTS_PROPS)
			state = state.setValue(prop, false)

		return state
	}

	override fun <T : BlockEntity> getTicker(
		level: Level,
		p_153213_: BlockState,
		type: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (level.isClientSide || type !== MBlockEntities.BATTERY_BANK)
			return null

		return BlockEntityTicker { _, _, _, tile -> if (tile is BlockEntityBatteryBank) tile.tick() }
	}

	override fun createBlockStateDefinition(builder: StateDefinition.Builder<Block, BlockState>) {
		builder.add(*BATTERY_SLOTS_PROPS)
		super.createBlockStateDefinition(builder)
	}

	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity? {
		return BlockEntityBatteryBank(blockPos, blockState)
	}

	override fun getShape(
		p_60555_: BlockState,
		p_60556_: BlockGetter,
		p_60557_: BlockPos,
		p_60558_: CollisionContext
	): VoxelShape {
		return SHAPES[p_60555_.getValue(FACING).ordinal]
	}

	override fun faceToPlayer(context: BlockPlaceContext): Boolean {
		return false
	}

	companion object {
		@JvmField
		val BATTERY_SLOTS_PROPS = arrayOf(
			BooleanProperty.create("battery_0") as BooleanProperty,
			BooleanProperty.create("battery_1") as BooleanProperty,
			BooleanProperty.create("battery_2") as BooleanProperty,
			BooleanProperty.create("battery_3") as BooleanProperty,
			BooleanProperty.create("battery_4") as BooleanProperty,
			BooleanProperty.create("battery_5") as BooleanProperty,
			BooleanProperty.create("battery_6") as BooleanProperty,
			BooleanProperty.create("battery_7") as BooleanProperty,
			BooleanProperty.create("battery_8") as BooleanProperty,
			BooleanProperty.create("battery_9") as BooleanProperty,
			BooleanProperty.create("battery_10") as BooleanProperty,
			BooleanProperty.create("battery_11") as BooleanProperty,
		)

		private val SHAPES: List<VoxelShape>

		init {
			val def = BlockShapes.BATTERY_BANK.computeShape()

			SHAPES = java.util.List.of(
				def,
				def,
				def,
				BlockShapes.BATTERY_BANK.rotate(Direction.NORTH).computeShape(),
				BlockShapes.BATTERY_BANK.rotate(Direction.WEST).computeShape(),
				BlockShapes.BATTERY_BANK.rotate(Direction.EAST).computeShape()
			)
		}
	}
}