package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.level.block.state.properties.BooleanProperty
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraft.world.phys.shapes.VoxelShape
import ru.dbotthepony.mc.otm.block.entity.BlockEntityPatternStorage
import ru.dbotthepony.mc.otm.shapes.BlockShapes

class BlockPatternStorage : BlockMatteryRotatable(), EntityBlock {
	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity? {
		return BlockEntityPatternStorage(blockPos, blockState)
	}

	override fun createBlockStateDefinition(builder: StateDefinition.Builder<Block, BlockState>) {
		builder.add(*PATTERN_STORAGE_DISKS_PROPS)
		super.createBlockStateDefinition(builder)
	}

	override fun getStateForPlacement(context: BlockPlaceContext): BlockState? {
		var state = super.getStateForPlacement(context) ?: return null

		for (prop in PATTERN_STORAGE_DISKS_PROPS)
			state = state.setValue(prop, false)

		return state
	}

	override fun getShape(
		p_60555_: BlockState,
		p_60556_: BlockGetter,
		p_60557_: BlockPos,
		p_60558_: CollisionContext
	): VoxelShape {
		return SHAPES[p_60555_.getValue(FACING).ordinal]
	}

	companion object {
		val PATTERN_STORAGE_DISKS_PROPS = arrayOf(
			BooleanProperty.create("disk_0") as BooleanProperty,
			BooleanProperty.create("disk_1") as BooleanProperty,
			BooleanProperty.create("disk_2") as BooleanProperty,
			BooleanProperty.create("disk_3") as BooleanProperty,
			BooleanProperty.create("disk_4") as BooleanProperty,
			BooleanProperty.create("disk_5") as BooleanProperty,
			BooleanProperty.create("disk_6") as BooleanProperty,
			BooleanProperty.create("disk_7") as BooleanProperty
		)

		private val SHAPES: List<VoxelShape>

		init {
			val def = BlockShapes.PATTERN_STORAGE.computeShape()

			SHAPES = listOf(
				def,
				def,
				def,
				BlockShapes.PATTERN_STORAGE.rotate(Direction.NORTH).computeShape(),
				BlockShapes.PATTERN_STORAGE.rotate(Direction.WEST).computeShape(),
				BlockShapes.PATTERN_STORAGE.rotate(Direction.EAST).computeShape()
			)
		}
	}
}