package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.world.MenuProvider
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.phys.AABB
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities

class BlockEntityAndroidStation(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntityMatteryPowered(MBlockEntities.ANDROID_STATION, p_155229_, p_155230_), MenuProvider {

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return AndroidStationMenu(containerID, inventory, this)
	}

	override fun getDefaultDisplayName(): Component {
		return MACHINE_NAME
	}

	override val energy = WorkerEnergyStorage(this, STORAGE, MAX_IO)

	fun tick() {
		batteryChargeLoop()

		if (isBlockedByRedstone) return
		val level = level ?: return
		val x = blockPos.x.toDouble()
		val y = blockPos.y.toDouble()
		val z = blockPos.z.toDouble()

		for (ent in level.getEntitiesOfClass(LivingEntity::class.java, AABB(x, y, z, x + 1.0, y + 2.0, z + 1.0))) {
			ent.getCapability(MatteryCapability.ANDROID).ifPresent {
				if (!it.isAndroid())
					return@ifPresent

				val missing = it.missingPower

				if (missing > ImpreciseFraction.ZERO) {
					val extract = energy.extractEnergyInner(missing, true)

					if (extract > ImpreciseFraction.ZERO) {
						val received = it.receiveEnergyOuter(extract, false)
						energy.extractEnergyInner(received, false)
					}
				}
			}
		}
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.android_station")
		private val STORAGE = ImpreciseFraction(100_000)
		private val MAX_IO = ImpreciseFraction(250)
	}
}