package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.items.CapabilityItemHandler
import ru.dbotthepony.mc.otm.block.BlockCargoCrate
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.menu.MenuCargoCrate
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.set

class BlockEntityCargoCrate(
	p_155229_: BlockPos,
	p_155230_: BlockState
) : BlockEntityMattery(MBlockEntities.CARGO_CRATE, p_155229_, p_155230_) {
	val container = MatteryContainer(this::setChanged, CAPACITY)
	private var interactingPlayers = 0
	val handler = container.handler()

	fun onPlayerOpen() {
		if (interactingPlayers++ == 0) {
			level?.setBlock(blockPos, blockState.setValue(BlockCargoCrate.IS_OPEN, true), Block.UPDATE_CLIENTS)
		}
	}

	fun onPlayerClose() {
		if (--interactingPlayers == 0) {
			level?.setBlock(blockPos, blockState.setValue(BlockCargoCrate.IS_OPEN, false), Block.UPDATE_CLIENTS)
		}
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		handler.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		handler.revive()
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (cap === CapabilityItemHandler.ITEM_HANDLER_CAPABILITY)
			return handler.get().cast()

		return super.getCapability(cap, side)
	}

	public override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["slots"] = container.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		nbt.ifHas("slots", CompoundTag::class.java, container::deserializeNBT)
	}

	override fun getDefaultDisplayName(): Component {
		return NAME
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MenuCargoCrate(containerID, inventory, this)
	}

	companion object {
		private val NAME = TranslatableComponent("block.overdrive_that_matters.cargo_crate")
		const val CAPACITY = 9 * 6
	}
}