package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.SectionPos
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.phys.AABB
import ru.dbotthepony.mc.otm.block.BlockGravitationStabilizer
import ru.dbotthepony.mc.otm.block.BlockGravitationStabilizerLens
import ru.dbotthepony.mc.otm.block.BlockMatteryRotatable
import ru.dbotthepony.mc.otm.block.entity.blackhole.BlockEntityBlackHole
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.core.plus
import ru.dbotthepony.mc.otm.core.times
import ru.dbotthepony.mc.otm.registry.MBlockEntities

class BlockEntityGravitationStabilizer(p_155229_: BlockPos, p_155230_: BlockState) : BlockEntityMattery(
	MBlockEntities.GRAVITATION_STABILIZER, p_155229_, p_155230_) {
	override fun getDefaultDisplayName() = NAME
	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player) = null

	private var blackHole: BlockEntityBlackHole? = null

	fun tick(level: Level) {
		var findBlackHole: BlockEntityBlackHole? = null
		val dir = blockState.getValue(BlockMatteryRotatable.FACING_FULL).normal

		for (i in 2 .. RANGE) {
			val pos = blockPos + dir * i
			val chunk = level.chunkSource.getChunkNow(SectionPos.blockToSectionCoord(pos.x), SectionPos.blockToSectionCoord(pos.z)) ?: continue
			val state = chunk.getBlockState(pos)

			if (!state.isAir) {
				val tile = chunk.getBlockEntity(pos)

				if (tile != null) {
					if (tile is BlockEntityBlackHole) {
						findBlackHole = tile
					}

					break
				}

				if (state.isViewBlocking(level, pos)) {
					break
				}
			}
		}

		if (findBlackHole != blackHole) {
			blackHole?.stabilizerDetached(this)
			findBlackHole?.stabilizerAttached(this)
			blackHole = findBlackHole

			level.setBlock(blockPos, blockState.setValue(WorkerState.SEMI_WORKER_STATE, if (findBlackHole != null) WorkerState.WORKING else WorkerState.IDLE), Block.UPDATE_CLIENTS)
			val posLens = BlockGravitationStabilizer.getBoundingBlockPos(blockState, blockPos)
			val stateLens = level.getBlockState(posLens)

			if (stateLens.block is BlockGravitationStabilizerLens) {
				level.setBlock(posLens, stateLens.setValue(WorkerState.SEMI_WORKER_STATE, if (findBlackHole != null) WorkerState.WORKING else WorkerState.IDLE), Block.UPDATE_CLIENTS)
			}
		}
	}

	override fun setRemoved() {
		super.setRemoved()
		blackHole?.stabilizerDetached(this)
	}

	override fun getRenderBoundingBox(): AABB {
		return AABB(blockPos.offset(-RANGE, -RANGE, -RANGE), blockPos.offset(RANGE, RANGE, RANGE))
	}

	companion object {
		private val minBBox = BlockPos(1, 1, 1)
		const val RANGE = 64
		private val NAME = TranslatableComponent("block.overdrive_that_matters.gravitation_stabilizer")
	}
}
