package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.graph.storage.BasicStorageGraphNode
import ru.dbotthepony.mc.otm.graph.storage.StorageNetworkGraph
import ru.dbotthepony.mc.otm.menu.MenuItemMonitor
import ru.dbotthepony.mc.otm.registry.MBlockEntities

class BlockEntityItemMonitor(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntityMatteryPowered(MBlockEntities.ITEM_MONITOR, p_155229_, p_155230_) {

	@JvmField
	val cell = BasicStorageGraphNode()
	override val energy = WorkerEnergyStorage(this)

	fun tick() {
		batteryChargeLoop()
	}

	override fun getDefaultDisplayName(): Component {
		return NAME
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MenuItemMonitor(containerID, inventory, this)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			StorageNetworkGraph.discoverFull(this, cell.getAsStorageNode())
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		return if (cap === MatteryCapability.STORAGE_NODE) {
			cell.get().cast()
		} else super.getCapability(cap, side)
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		cell.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		cell.revive()
	}

	companion object {
		private val NAME = TranslatableComponent("block.overdrive_that_matters.item_monitor")
	}
}