package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.items.CapabilityItemHandler
import net.minecraftforge.items.IItemHandler
import ru.dbotthepony.mc.otm.block.entity.worker.BlockEntityMatteryWorker
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerJob
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerJobStatus
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler
import ru.dbotthepony.mc.otm.capability.matter.MatterDirection
import ru.dbotthepony.mc.otm.capability.matter.MatterHandlerImpl
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.graph.Graph6Node
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.matter.baselineComplexityDecomposeTicks
import ru.dbotthepony.mc.otm.matter.canDecompose
import ru.dbotthepony.mc.otm.matter.getMatterValue
import ru.dbotthepony.mc.otm.menu.MenuMatterDecomposer
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.set

fun moveMatterAsDustIntoContainer(_matterValue: ImpreciseFraction, container: MatteryContainer, OUTPUT_DUST_MAIN: Int, OUTPUT_DUST_STACKING: Int): ImpreciseFraction {
	var matterValue = _matterValue
	val item = MItems.MATTER_DUST

	while (matterValue > ImpreciseFraction.ZERO) {
		val stack = container[OUTPUT_DUST_MAIN]

		// первый слот пустой
		if (stack.isEmpty) {
			container[OUTPUT_DUST_MAIN] = ItemStack(item, 1).also {
				matterValue -= item.addMatterValue(it, matterValue, false)
			}
			// первый слот не пустой, но мы можем влить туда материю
		} else if (!item.isFull(stack) && stack.count == 1) {
			matterValue -= item.addMatterValue(stack, matterValue, false)
			container.setChanged(OUTPUT_DUST_MAIN)
			// первый слот не пустой и мы не можем влить туда материю
		} else {
			val stack2 = container[OUTPUT_DUST_STACKING]

			// второй слот пустой
			if (stack2.isEmpty) {
				container[OUTPUT_DUST_STACKING] = ItemStack(item, 1).also {
					matterValue -= item.addMatterValue(it, matterValue, false)
				}
				// второй слот не пустой, но мы можем влить туда материю
			} else if (!item.isFull(stack2)) {
				if (stack2.count != 1) {
					return matterValue
				}

				matterValue -= item.addMatterValue(stack2, matterValue, false)
				container.setChanged(OUTPUT_DUST_STACKING)
			}

			// можем ли мы стакнуть материю из второго слота в первый?
			if (!stack2.isEmpty && item.isFull(stack2)) {
				if (ItemStack.isSameItemSameTags(stack, stack2) && container.getMaxStackSizeWithItem(OUTPUT_DUST_MAIN) >= stack.count + 1) {
					stack.count++
					stack2.count--

					container.setChanged(OUTPUT_DUST_MAIN)
					container.setChanged(OUTPUT_DUST_STACKING)
				} else {
					return matterValue
				}
			}
		}
	}

	return ImpreciseFraction.ZERO
}

class BlockEntityMatterDecomposer(pos: BlockPos, state: BlockState)
	: BlockEntityMatteryWorker(MBlockEntities.MATTER_DECOMPOSER, pos, state), IMatterGraphNode {

	override val energy = WorkerEnergyStorage(this, ENERGY_STORAGE, MAX_IO)
	private var valid = true
	private val node = Graph6Node<IMatterGraphNode>(this)

	override fun getAsMatterNode(): Graph6Node<IMatterGraphNode> {
		return node
	}

	@JvmField
	val matter = MatterHandlerImpl(
		this::setChangedLight,
		MatterDirection.EXTRACT,
		CAPACITY
	)

	private var resolverMatter = LazyOptional.of { matter }
	private var resolverNode = LazyOptional.of { this }

	// вход, выход
	@JvmField
	val container = MatteryContainer(this::setChangedLight, 3)

	private val itemHandler = LazyOptional.of<IItemHandler> {
		container.handler(
			{ slot: Int, stack: ItemStack -> slot == INPUT_SLOT && canDecompose(stack) },
			{ slot: Int, _: Int, _: ItemStack -> slot != INPUT_SLOT })
	}

	override fun getDefaultDisplayName(): Component {
		return MACHINE_NAME
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MenuMatterDecomposer(containerID, inventory, this)
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["container"] = container.serializeNBT()
		nbt["matter"] = matter.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)

		nbt.ifHas("matter", CompoundTag::class.java) {
			matter.deserializeNBT(it)
		}

		container.deserializeNBT(nbt["container"])
	}

	override fun reviveCaps() {
		valid = true
		super.reviveCaps()
		resolverMatter = LazyOptional.of { matter }
	}

	override fun invalidateCaps() {
		valid = false
		super.invalidateCaps()
		resolverMatter.invalidate()
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === MatteryCapability.MATTER) return resolverMatter.cast()
			if (cap === MatteryCapability.MATTER_NODE) return resolverNode.cast()
			if (cap === CapabilityItemHandler.ITEM_HANDLER_CAPABILITY) return itemHandler.cast()
		}

		return super.getCapability(cap, side)
	}

	override fun onJobFinish(job: WorkerJob): WorkerJobStatus {
		var matterValue = ImpreciseFraction.deserializeNBT(job.data["value"])

		if (job.data.getBoolean("to_dust")) {
			matterValue = moveMatterAsDustIntoContainer(matterValue, container, OUTPUT_DUST_MAIN, OUTPUT_DUST_STACKING)

			if (!matterValue.isZero) {
				job.data["value"] = matterValue.serializeNBT()
				return WorkerJobStatus(20)
			}

			return WorkerJobStatus()
		}

		matterValue -= matter.receiveMatterInner(matterValue, false)

		if (matterValue.isPositive) {
			job.data["value"] = matterValue.serializeNBT()
			return WorkerJobStatus(false, 20)
		}

		return WorkerJobStatus()
	}

	override fun computeNextJob(): WorkerJob? {
		val stack = container[INPUT_SLOT]

		if (!stack.isEmpty) {
			val copy = stack.copy()
			copy.count = 1

			if (canDecompose(copy)) {
				val matter = getMatterValue(copy)

				if (!matter.isZero) {
					stack.count--

					return WorkerJob(copy, matter.complexity * baselineComplexityDecomposeTicks, BASE_CONSUMPTION, CompoundTag().also {
						it["to_dust"] = (level?.random?.nextDouble() ?: 1.0) <= 0.2
						it["value"] = matter.value.serializeNBT()
					})
				}
			}
		}

		return null
	}

	override fun setRemoved() {
		super.setRemoved()
		node.destroy(::MatterNetworkGraph)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			MatterNetworkGraph.discoverFull(this, node)
	}

	override fun getMatterHandler(): IMatterHandler {
		return matter
	}

	fun tick() {
		batteryChargeLoop()
		workerLoop()

		val grid = node.graph as MatterNetworkGraph? ?: return

		if (!matter.storedMatter.isZero) {
			val diff = matter.extractMatterInner(matter.storedMatter, true)
			val diff2 = grid.receiveMatter(diff, true)

			matter.extractMatterInner(diff2, false)
			grid.receiveMatter(diff2, false)
		}
	}

	companion object {
		const val INPUT_SLOT = 0
		const val OUTPUT_DUST_MAIN = 1
		const val OUTPUT_DUST_STACKING = 2

		private val CAPACITY = ImpreciseFraction("20")
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.matter_decomposer")
		private val BASE_CONSUMPTION = ImpreciseFraction(240)

		private val ENERGY_STORAGE = ImpreciseFraction(400_000)
		private val MAX_IO = ImpreciseFraction(2_000)
	}
}
