package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.items.CapabilityItemHandler
import ru.dbotthepony.mc.otm.block.entity.worker.BlockEntityMatteryWorker
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerJob
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerJobStatus
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.container.MatteryContainerFilter
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.menu.MenuPlatePress
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.registry.MRecipes
import ru.dbotthepony.mc.otm.set

class BlockEntityPlatePress(p_155229_: BlockPos, p_155230_: BlockState) : BlockEntityMatteryWorker(MBlockEntities.PLATE_PRESS, p_155229_, p_155230_) {
	val container = MatteryContainer(this::setChangedLight, 2)
	override val energy = WorkerEnergyStorage(this::setChangedLight)

	val itemHandler = container.handler(object : MatteryContainerFilter {
		override fun canInsert(slot: Int, stack: ItemStack): Boolean {
			return slot != SLOT_OUTPUT
		}

		override fun canExtract(slot: Int, amount: Int, stack: ItemStack): Boolean {
			return slot != SLOT_INPUT
		}
	})

	override fun invalidateCaps() {
		super.invalidateCaps()
		itemHandler.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		itemHandler.revive()
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (cap === CapabilityItemHandler.ITEM_HANDLER_CAPABILITY)
			return itemHandler.get().cast()

		return super.getCapability(cap, side)
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["container"] = container.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		container.deserializeNBT(nbt["container"])
	}

	override fun getDefaultDisplayName() = NAME

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MenuPlatePress(containerID, inventory, this)
	}

	override fun onJobFinish(job: WorkerJob): WorkerJobStatus {
		val resultTag = job.data["result"] as? CompoundTag ?: return WorkerJobStatus()
		val result = ItemStack.of(resultTag)

		if (result.isEmpty)
			return WorkerJobStatus()

		if (!container.fullyAddItem(result, start = SLOT_OUTPUT, end = SLOT_OUTPUT)) {
			return WorkerJobStatus(false, 20)
		}

		return WorkerJobStatus()
	}

	override fun computeNextJob(): WorkerJob? {
		val level = level ?: return null
		val recipe = level.recipeManager.getRecipeFor(MRecipes.PLATE_PRESS, container, level).orElse(null) ?: return null

		val copy = container[SLOT_INPUT].copy()
		container[SLOT_INPUT].shrink(1)
		container.setChanged(SLOT_INPUT)
		copy.count = 1

		return WorkerJob(copy, recipe.workTime.toDouble(), BASELINE_CONSUMPTION, CompoundTag().also {
			it["result"] = recipe.resultItem.serializeNBT()
		})
	}

	companion object {
		private val BASELINE_CONSUMPTION = ImpreciseFraction(15)
		private val NAME = TranslatableComponent("block.overdrive_that_matters.plate_press")
		const val SLOT_INPUT = 0
		const val SLOT_OUTPUT = 1
	}
}
