package ru.dbotthepony.mc.otm.capability

import net.minecraftforge.energy.IEnergyStorage
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.core.ImpreciseFraction

// IEnergyStorage for direct compat with Forge Energy
interface IMatteryEnergyStorage : IEnergyStorage {
	// such as cables. This is something that would work only with energy storage
	fun extractEnergyOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction

	// for internal needs, e.g. for work
	// CAN also be used by something that does evil
	// e.g. sucking out energy anomaly should use this
	fun extractEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction

	// energy is received from outside, e.g. cables
	fun receiveEnergyOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction

	// energy is received from inside, e.g. generator generates power
	fun receiveEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction

	fun extractEnergyOuter(howMuch: Long, simulate: Boolean): ImpreciseFraction {
		return extractEnergyOuter(ImpreciseFraction(howMuch), simulate)
	}

	fun extractEnergyOuter(howMuch: Int, simulate: Boolean): ImpreciseFraction {
		return extractEnergyOuter(ImpreciseFraction(howMuch), simulate)
	}

	fun receiveEnergyOuter(howMuch: Long, simulate: Boolean): ImpreciseFraction {
		return receiveEnergyOuter(ImpreciseFraction(howMuch), simulate)
	}

	fun receiveEnergyOuter(howMuch: Int, simulate: Boolean): ImpreciseFraction {
		return receiveEnergyOuter(ImpreciseFraction(howMuch), simulate)
	}

	fun extractEnergyInner(howMuch: Long, simulate: Boolean): ImpreciseFraction {
		return extractEnergyInner(ImpreciseFraction(howMuch), simulate)
	}

	fun extractEnergyInner(howMuch: Int, simulate: Boolean): ImpreciseFraction {
		return extractEnergyInner(ImpreciseFraction(howMuch), simulate)
	}

	fun receiveEnergyInner(howMuch: Long, simulate: Boolean): ImpreciseFraction {
		return receiveEnergyInner(ImpreciseFraction(howMuch), simulate)
	}

	fun receiveEnergyInner(howMuch: Int, simulate: Boolean): ImpreciseFraction {
		return receiveEnergyInner(ImpreciseFraction(howMuch), simulate)
	}

	val batteryLevel: ImpreciseFraction
	val maxBatteryLevel: ImpreciseFraction
	val missingPower: ImpreciseFraction
		get() = (maxBatteryLevel - batteryLevel).moreThanZero()

	override fun receiveEnergy(maxReceive: Int, simulate: Boolean): Int {
		val received = receiveEnergyOuter(maxReceive, true).toInt()

		// Receiving only a fraction
		if (received == 0)
			return 0

		return receiveEnergyOuter(ImpreciseFraction(received), simulate).toInt()
	}

	override fun extractEnergy(maxReceive: Int, simulate: Boolean): Int {
		val extracted = extractEnergyOuter(maxReceive, true).toInt()

		// Extracting only a fraction
		if (extracted == 0)
			return 0

		return extractEnergyOuter(ImpreciseFraction(extracted), simulate).toInt()
	}

	override fun getEnergyStored(): Int {
		return batteryLevel.toInt()
	}

	override fun getMaxEnergyStored(): Int {
		return maxBatteryLevel.toInt()
	}

	override fun canExtract(): Boolean {
		return extractEnergyOuter(ImpreciseFraction.ONE, true) > ImpreciseFraction.ZERO
	}

	override fun canReceive(): Boolean {
		return receiveEnergyOuter(ImpreciseFraction.ONE, true) > ImpreciseFraction.ZERO
	}
}