package ru.dbotthepony.mc.otm.capability.matter

import net.minecraft.nbt.CompoundTag
import net.minecraftforge.common.util.INBTSerializable
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.set

open class MatterHandlerImpl @JvmOverloads constructor(
	protected val listener: Runnable?,
	override val direction: MatterDirection,
	override val maxStoredMatter: ImpreciseFraction,
	val maxReceive: ImpreciseFraction? = null,
	val maxExtract: ImpreciseFraction? = maxReceive
) : IMatterHandler, INBTSerializable<CompoundTag> {
	override var storedMatter = ImpreciseFraction.ZERO
		protected set

	private var handler = LazyOptional.of<IMatterHandler> { this }

	fun invalidate() {
		handler.invalidate()
	}

	fun revive() {
		handler = LazyOptional.of { this }
	}

	fun get(): LazyOptional<IMatterHandler> {
		return handler
	}

	open fun canReceiveAll(value: ImpreciseFraction): Boolean {
		return maxStoredMatter >= value && storedMatter + value <= maxStoredMatter
	}

	override fun receiveMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		if (direction === MatterDirection.EXTRACT)
			return ImpreciseFraction.ZERO

		return receiveMatterInner(howMuch, simulate)
	}

	override fun receiveMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		val new: ImpreciseFraction

		if (maxReceive == null) {
			new = (storedMatter + howMuch).min(maxStoredMatter)
		} else {
			new = (storedMatter + howMuch.min(maxReceive)).min(maxStoredMatter)
		}

		val diff = new - storedMatter

		if (!simulate && new != storedMatter) {
			storedMatter = new
			listener?.run()
		}

		return diff
	}

	override fun extractMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		if (direction === MatterDirection.RECEIVE)
			return ImpreciseFraction.ZERO

		return extractMatterInner(howMuch, simulate)
	}

	override fun extractMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		val new: ImpreciseFraction

		if (maxExtract == null) {
			new = (storedMatter - howMuch).moreThanZero()
		} else {
			new = (storedMatter - howMuch.min(maxExtract)).moreThanZero()
		}

		val diff = storedMatter - new

		if (!simulate && new != storedMatter) {
			storedMatter = new
			listener?.run()
		}

		return diff
	}

	override fun serializeNBT(): CompoundTag {
		return CompoundTag().also {
			it["stored"] = storedMatter.serializeNBT()
			//it["max_storage"] = maxStoredMatter.serializeNBT()
			//if (maxReceive != null) it["max_receive"] = maxReceive.serializeNBT()
			//if (maxExtract != null) it["max_extract"] = maxExtract.serializeNBT()
		}
	}

	override fun deserializeNBT(tag: CompoundTag) {
		storedMatter = ImpreciseFraction.deserializeNBT(tag["stored"])
		//maxStoredMatter = ImpreciseFraction.deserializeNBT(tag["max_storage"])
		//maxReceive = if (tag.contains("max_receive")) ImpreciseFraction.deserializeNBT(tag["max_receive"]) else null
		//maxExtract = if (tag.contains("max_extract")) ImpreciseFraction.deserializeNBT(tag["max_extract"]) else null
	}
}