package ru.dbotthepony.mc.otm.item

import ru.dbotthepony.mc.otm.core.Fraction.Companion.deserializeNBT
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage
import net.minecraftforge.common.capabilities.ICapabilityProvider
import net.minecraft.world.item.ItemStack
import net.minecraft.nbt.CompoundTag
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.energy.CapabilityEnergy
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import net.minecraft.network.chat.TranslatableComponent
import ru.dbotthepony.mc.otm.menu.FormattingHelper
import net.minecraft.ChatFormatting
import net.minecraft.core.Direction
import net.minecraft.network.chat.Component
import net.minecraft.world.item.Item
import net.minecraft.world.item.Rarity
import net.minecraft.world.item.TooltipFlag
import net.minecraft.world.level.Level
import net.minecraftforge.common.capabilities.Capability
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.core.ImpreciseFraction

class ItemBattery : Item {
	private inner class BatteryMatteryCapability(private val stack: ItemStack) : IMatteryEnergyStorage, ICapabilityProvider {
		private fun energy(): ImpreciseFraction {
			val tag = stack.orCreateTag
			return if (tag.contains("otm_energy")) {
				ImpreciseFraction.deserializeNBT(tag["otm_energy"])
			} else ImpreciseFraction.ZERO
		}

		private fun energy(value: ImpreciseFraction) {
			stack.getOrCreateTag().put("otm_energy", value.serializeNBT())
		}

		override fun extractEnergyOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			return extractEnergyInner(howMuch, simulate)
		}

		override fun extractEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			if (isCreative) return howMuch
			val new = energy().minus(howMuch.min(extract)).moreThanZero()
			val diff = energy().minus(new)

			if (!simulate) {
				energy(new)
			}

			return diff
		}

		override fun receiveEnergyOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			return receiveEnergyInner(howMuch, simulate)
		}

		override fun receiveEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			if (isCreative) return howMuch
			val new = energy().plus(howMuch.min(receive)).min(storage)
			val diff = new.minus(energy())

			if (!simulate) {
				energy(new)
			}

			return diff
		}

		override val missingPower: ImpreciseFraction get() {
			return if (isCreative) ImpreciseFraction.LONG_MAX_VALUE else super.missingPower
		}

		override val batteryLevel: ImpreciseFraction get() {
			return if (isCreative) ImpreciseFraction.LONG_MAX_VALUE else energy()
		}

		override val maxBatteryLevel: ImpreciseFraction get() {
			return storage
		}

		private val resolver = LazyOptional.of<IMatteryEnergyStorage> { this }

		override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
			return if (cap === MatteryCapability.ENERGY || cap === CapabilityEnergy.ENERGY) resolver.cast() else LazyOptional.empty()
		}

		override fun canExtract(): Boolean {
			return true
		}

		override fun canReceive(): Boolean {
			return true
		}
	}

	private val isCreative: Boolean
	val storage: ImpreciseFraction
	val receive: ImpreciseFraction
	val extract: ImpreciseFraction
	private val throughputText: Component

	constructor(storage: ImpreciseFraction, receive: ImpreciseFraction, extract: ImpreciseFraction) : super(
		Properties().stacksTo(1).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB)
	) {
		isCreative = false
		this.storage = storage
		this.receive = receive
		this.extract = extract
		throughputText = TranslatableComponent(
			"otm.item.power.normal.throughput",
			FormattingHelper.formatPower(receive),
			FormattingHelper.formatPower(extract)
		).withStyle(ChatFormatting.GRAY)
	}

	constructor() : super(Properties().stacksTo(1).rarity(Rarity.EPIC).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB)) {
		isCreative = true
		storage = ImpreciseFraction.LONG_MAX_VALUE
		receive = ImpreciseFraction.LONG_MAX_VALUE
		extract = ImpreciseFraction.LONG_MAX_VALUE
		throughputText = TranslatableComponent("otm.item.power.infinite.throughput").withStyle(ChatFormatting.GRAY)
	}

	override fun appendHoverText(
		stack: ItemStack,
		p_41422_: Level?,
		p_41423_: MutableList<Component>,
		p_41424_: TooltipFlag
	) {
		super.appendHoverText(stack, p_41422_, p_41423_, p_41424_)
		if (isCreative) {
			p_41423_.add(INFINITE_STORAGE)
			p_41423_.add(throughputText)
		} else {
			stack.getCapability(MatteryCapability.ENERGY).ifPresent {
				p_41423_.add(
					TranslatableComponent(
						"otm.item.power.normal.storage",
						FormattingHelper.formatPower(it.batteryLevel),
						FormattingHelper.formatPower(it.maxBatteryLevel)
					).withStyle(ChatFormatting.GRAY)
				)
			}
			p_41423_.add(throughputText)
		}
	}

	override fun initCapabilities(stack: ItemStack, nbt: CompoundTag?): ICapabilityProvider {
		return BatteryMatteryCapability(stack)
	}

	companion object {
		private val INFINITE_STORAGE: Component =
			TranslatableComponent("otm.item.power.infinite.storage").withStyle(ChatFormatting.GRAY)
	}
}