package ru.dbotthepony.mc.otm.item

import ru.dbotthepony.mc.otm.OverdriveThatMatters
import net.minecraft.world.item.ItemStack
import net.minecraft.nbt.CompoundTag
import net.minecraftforge.common.capabilities.ICapabilityProvider
import net.minecraft.world.item.TooltipFlag
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.ChatFormatting
import net.minecraftforge.common.util.LazyOptional
import net.minecraft.nbt.ListTag
import net.minecraft.core.Direction
import net.minecraft.network.chat.Component
import net.minecraft.world.item.Item
import net.minecraft.world.level.Level
import net.minecraftforge.common.capabilities.Capability
import ru.dbotthepony.mc.otm.capability.matter.*
import ru.dbotthepony.mc.otm.ifHas
import java.util.*

class ItemPatternStorage : Item {
	val capacity: Int
	var isCreative: Boolean

	constructor(capacity: Int) : super(Properties().tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB).stacksTo(1)) {
		this.capacity = capacity
		isCreative = false
	}

	constructor() : super(Properties().tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB).stacksTo(1)) {
		isCreative = true
		capacity = Int.MAX_VALUE
	}

	override fun initCapabilities(stack: ItemStack, nbt: CompoundTag?): ICapabilityProvider? {
		return ItemPatternStorageCapability(stack)
	}

	override fun appendHoverText(
		p_41421_: ItemStack,
		p_41422_: Level?,
		list: MutableList<Component>,
		p_41424_: TooltipFlag
	) {
		p_41421_.getCapability(MatteryCapability.PATTERN).ifPresent {
			if (isCreative)
				list.add(TranslatableComponent("otm.item.pattern.infinite.stored", it.stored).withStyle(ChatFormatting.GRAY))
			else
				list.add(TranslatableComponent("otm.item.pattern.stored", it.stored, it.capacity).withStyle(ChatFormatting.GRAY))

			for (state in it.storedPatterns) {
				list.add(
					TranslatableComponent(
						"otm.item.pattern.line",
						state.item().getName(ItemStack(state.item(), 1)),
						String.format("%.2f", state.research() * 100.0)
					).withStyle(ChatFormatting.AQUA)
				)
			}
		}

		super.appendHoverText(p_41421_, p_41422_, list, p_41424_)
	}

	private inner class ItemPatternStorageCapability(val stack: ItemStack) : IPatternStorage, ICapabilityProvider {
		private val resolver = LazyOptional.of<IPatternStorage> { this }

		override val capacity: Int get() {
			return this@ItemPatternStorage.capacity
		}

		override val stored: Int get() {
			return storedPatterns.size
		}

		override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
			return if (cap === MatteryCapability.PATTERN) resolver.cast() else LazyOptional.empty()
		}

		override val storedPatterns: Collection<PatternState> get() {
			stack.tag?.ifHas("otm_patterns", ListTag::class.java) {
				val list = ArrayList<PatternState>()

				for (tag in it) {
					if (tag is CompoundTag) {
						val state = PatternState.deserializeNBT(tag)

						if (state != null)
							list.add(state)
					}
				}

				return list
			}

			return emptyList()
		}

		override fun insertPattern(
			pattern: PatternState,
			only_update: Boolean,
			simulate: Boolean
		): PatternInsertStatus {
			val tag = stack.orCreateTag
			var list = tag["otm_patterns"]

			if (list !is ListTag) {
				if (only_update)
					return PatternInsertFailure

				if (simulate) {
					if (capacity > 0)
						return PatternInsertInserted(pattern)
					else
						return PatternInsertFailure
				}

				list = ListTag()
				tag.put("otm_patterns", list)
			}

			var invalidCounter = 0

			for (i in list.indices) {
				val state = PatternState.deserializeNBT(list.getCompound(i))

				if (state != null) {
					if (state == pattern) {
						if (!simulate) {
							list[i] = pattern.serializeNBT()
						}

						return PatternInsertUpdated(pattern, state)
					}
				} else {
					invalidCounter++
				}
			}

			if (only_update || capacity <= list.size - invalidCounter)
				return PatternInsertFailure

			if (invalidCounter > 0) {
				if (simulate)
					return PatternInsertInserted(pattern)

				for (i in list.indices) {
					val state = PatternState.deserializeNBT(list.getCompound(i))

					if (state == null) {
						list[i] = pattern.serializeNBT()
						return PatternInsertInserted(pattern)
					}
				}
			}

			if (!simulate)
				list.add(pattern.serializeNBT())

			return PatternInsertInserted(pattern)
		}
	}
}