package ru.dbotthepony.mc.otm.menu

import kotlin.jvm.JvmOverloads
import net.minecraft.world.entity.player.Inventory
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterDecomposer
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import net.minecraft.world.SimpleContainer
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.matter.canDecompose
import ru.dbotthepony.mc.otm.orNull
import ru.dbotthepony.mc.otm.registry.MMenus

class MenuMatterDecomposer @JvmOverloads constructor(
	containerID: Int,
	inventory: Inventory,
	tile: BlockEntityMatterDecomposer? = null
) : MenuMatteryPowered(MMenus.MATTER_DECOMPOSER, containerID, inventory, tile) {
	val input: MatterySlot
	val outputMain: MachineOutputSlot
	val outputStacking: MachineOutputSlot
	val progressWidget: ProgressGaugeWidget
	val matterWidget: LevelGaugeWidget

	init {
		val container = tile?.container ?: SimpleContainer(3)

		// Вход
		input = object : MatterySlot(container, 0) {
			override fun mayPlace(p_40231_: ItemStack) = canDecompose(p_40231_)
		}

		addSlot(input)

		// Выход
		outputMain = MachineOutputSlot(container, 1)
		outputStacking = MachineOutputSlot(container, 2)

		addSlot(outputMain)
		addSlot(outputStacking)

		matterWidget = LevelGaugeWidget(this, tile?.getCapability(MatteryCapability.MATTER)?.orNull())

		if (tile == null) {
			progressWidget = ProgressGaugeWidget(this)
		} else {
			progressWidget = ProgressGaugeWidget(this, tile::workProgress, tile::isUnableToProcess)
		}

		addInventorySlots()
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 3
	}
}